@@ -1,4 +1,4 @@
-/* $Id: browser.c,v 1.42 2006/05/21 21:23:21 dolorous Exp $ */
+/* $Id: browser.c,v 1.112 2006/08/26 15:46:20 dolorous Exp $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
@@ -31,11 +31,11 @@
 #ifndef DISABLE_BROWSER
 
 static char **filelist = NULL;
-	/* The list of files to display in the browser. */
+	/* The list of files to display in the file browser. */
 static size_t filelist_len = 0;
 	/* The number of files in the list. */
 static int width = 0;
-	/* The number of columns to display the list in. */
+	/* The number of files that we can display per line. */
 static int longest = 0;
 	/* The number of columns in the longest filename in the list. */
 static size_t selected = 0;
@@ -44,16 +44,22 @@ static size_t selected = 0;
 static bool search_last_file = FALSE;
 	/* Have we gone past the last file while searching? */
 
-/* Our browser function.  path is the tilde-expanded path to start
- * browsing from. */
+/* Our main file browser function.  path is the tilde-expanded path to
+ * start browsing from. */
 char *do_browser(char *path, DIR *dir)
 {
-    int kbinput;
-    bool meta_key, func_key;
-    bool old_const_update = ISSET(CONST_UPDATE);
-    char *ans = mallocstrcpy(NULL, "");
-	/* The last answer the user typed on the statusbar. */
     char *retval = NULL;
+    int kbinput;
+    bool meta_key, func_key, old_const_update = ISSET(CONST_UPDATE);
+    bool abort = FALSE;
+	/* Whether we should abort the file browser. */
+    char *prev_dir = NULL;
+	/* The directory we were in, if any, before backing up via
+	 * browsing to "..". */
+    char *ans = NULL;
+	/* The last answer the user typed on the statusbar. */
+    size_t old_selected;
+	/* The selected file we had before the current selected file. */
 
     curs_set(0);
     blank_statusbar();
@@ -65,86 +71,99 @@ char *do_browser(char *path, DIR *dir)
 
     UNSET(CONST_UPDATE);
 
-  change_browser_directory:
-	/* We go here after the user selects a new directory. */
+    ans = mallocstrcpy(NULL, "");
 
+  change_browser_directory:
+	/* We go here after we select a new directory. */
+
+    /* Start with no key pressed. */
     kbinput = ERR;
-    meta_key = FALSE;
-    func_key = FALSE;
-    width = 0;
-    selected = 0;
 
     path = mallocstrassn(path, get_full_path(path));
 
-    /* Assume that path exists and ends with a slash. */
     assert(path != NULL && path[strlen(path) - 1] == '/');
 
-    /* Get the list of files. */
+    /* Get the file list, and set longest and width in the process. */
     browser_init(path, dir);
 
     assert(filelist != NULL);
 
-    /* Sort the list. */
+    /* Sort the file list. */
     qsort(filelist, filelist_len, sizeof(char *), diralphasort);
 
+    /* If prev_dir isn't NULL, select the directory saved in it, and
+     * then blow it away. */
+    if (prev_dir != NULL) {
+	browser_select_filename(prev_dir);
+
+	free(prev_dir);
+	prev_dir = NULL;
+    /* Otherwise, select the first file or directory in the list. */
+    } else
+	selected = 0;
+
+    old_selected = (size_t)-1;
+
     titlebar(path);
 
-    do {
-	bool abort = FALSE;
+    while (!abort) {
 	struct stat st;
 	int i;
-	size_t fileline;
+	size_t fileline = selected / width;
+		/* The line number the selected file is on. */
 	char *new_path;
-	    /* Used by the "Go To Directory" prompt. */
-#ifndef DISABLE_MOUSE
-	MEVENT mevent;
-#endif
+		/* The path we switch to at the "Go to Directory"
+		 * prompt. */
 
-	/* Compute the line number we're on now, so that we don't divide
-	 * by zero. */
-	fileline = selected;
-	if (width != 0)
-	    fileline /= width;
+	/* Display the file list if we don't have a key, or if the
+	 * selected file has changed, and set width in the process. */
+	if (kbinput == ERR || old_selected != selected)
+	    browser_refresh();
+
+	old_selected = selected;
+
+	kbinput = get_kbinput(edit, &meta_key, &func_key);
+	parse_browser_input(&kbinput, &meta_key, &func_key);
 
 	switch (kbinput) {
 #ifndef DISABLE_MOUSE
 	    case KEY_MOUSE:
-		if (getmouse(&mevent) == ERR)
-		    break;
-
-		/* If we clicked in the edit window, we probably clicked
-		 * on a file. */
-		if (wenclose(edit, mevent.y, mevent.x)) {
-		    size_t old_selected = selected;
-
-		    /* Subtract out the size of topwin. */
-		    mevent.y -= 2 - no_more_space();
-
-		    /* longest is the width of each column.  There are
-		     * two spaces between each column. */
-		    selected = (fileline / editwinrows) * (editwinrows *
-			width) + (mevent.y * width) + (mevent.x /
-			(longest + 2));
-
-		    /* If they clicked beyond the end of a row, select
-		     * the end of that row. */
-		    if (mevent.x > width * (longest + 2))
-			selected--;
-
-		    /* If we're off the screen, reset to the last item.
-		     * If we clicked the same place as last time, select
-		     * this name! */
-		    if (selected > filelist_len - 1)
-			selected = filelist_len - 1;
-		    else if (old_selected == selected)
-			/* Put back the "Select" key, so that the file
-			 * is selected. */
-			unget_kbinput(NANO_ENTER_KEY, FALSE, FALSE);
-		} else {
-		    /* We must have clicked a shortcut.  Put back the
-		     * equivalent shortcut key. */
+		{
 		    int mouse_x, mouse_y;
-		    get_mouseinput(&mouse_x, &mouse_y, TRUE);
+
+		    if (!get_mouseinput(&mouse_x, &mouse_y, TRUE)) {
+			/* We can click in the edit window to select a
+			 * filename. */
+			if (wenclose(edit, mouse_y, mouse_x)) {
+			    /* Subtract out the size of topwin. */
+			    mouse_y -= 2 - no_more_space();
+
+			    /* longest is the width of each column.
+			     * There are two spaces between each
+			     * column. */
+			    selected = (fileline / editwinrows) *
+				(editwinrows * width) + (mouse_y *
+				width) + (mouse_x / (longest + 2));
+
+			    /* If they clicked beyond the end of a row,
+			     * select the filename at the end of that
+			     * row. */
+			    if (mouse_x > width * (longest + 2))
+				selected--;
+
+			    /* If we're off the screen, select the last
+			     * filename. */
+			    if (selected > filelist_len - 1)
+				selected = filelist_len - 1;
+
+			    /* If we selected the same filename as last
+			     * time, put back the Enter key so that it's
+			     * read in. */
+			    if (old_selected == selected)
+				unget_kbinput(NANO_ENTER_KEY, FALSE,
+					FALSE);
+			}
+		    }
 		}
 		break;
 #endif /* !DISABLE_MOUSE */
@@ -181,7 +200,7 @@ char *do_browser(char *path, DIR *dir)
 	    case NANO_NEXTPAGE_KEY:
 		selected += (editwinrows - fileline % editwinrows) *
 			width;
-		if (selected >= filelist_len)
+		if (selected > filelist_len - 1)
 		    selected = filelist_len - 1;
 		break;
 	    case NANO_FIRSTFILE_ALTKEY:
@@ -252,7 +271,7 @@ char *do_browser(char *path, DIR *dir)
 
 		dir = opendir(new_path);
 		if (dir == NULL) {
-		    /* We can't open this dir for some reason.
+		    /* We can't open this directory for some reason.
 		     * Complain. */
 		    statusbar(_("Error reading %s: %s"), answer,
 			strerror(errno));
@@ -264,7 +283,6 @@ char *do_browser(char *path, DIR *dir)
 		/* Start over again with the new path value. */
 		free(path);
 		path = new_path;
-		free_chararray(filelist, filelist_len);
 		goto change_browser_directory;
 	    case NANO_PREVLINE_KEY:
 		if (selected >= width)
@@ -283,7 +301,7 @@ char *do_browser(char *path, DIR *dir)
 		    selected++;
 		break;
 	    case NANO_ENTER_KEY:
-		/* You can't move up from "/". */
+		/* We can't move up from "/". */
 		if (strcmp(filelist[selected], "/..") == 0) {
 		    statusbar(_("Can't move up a directory"));
 		    beep();
@@ -291,7 +309,7 @@ char *do_browser(char *path, DIR *dir)
 		}
 
 #ifndef DISABLE_OPERATINGDIR
-		/* Note: the selected file can be outside the operating
+		/* Note: The selected file can be outside the operating
 		 * directory if it's ".." or if it's a symlink to a
 		 * directory outside the operating directory. */
 		if (check_operating_dir(filelist[selected], FALSE)) {
@@ -312,11 +330,18 @@ char *do_browser(char *path, DIR *dir)
 		    break;
 		}
 
+		/* If we've successfully opened a file, we're done, so
+		 * get out. */
 		if (!S_ISDIR(st.st_mode)) {
-		    retval = mallocstrcpy(retval, filelist[selected]);
+		    retval = mallocstrcpy(NULL, filelist[selected]);
 		    abort = TRUE;
 		    break;
-		}
+		/* If we've successfully opened a directory, and it's
+		 * "..", save the current directory in prev_dir, so that
+		 * we can select it later. */
+		} else if (strcmp(tail(filelist[selected]), "..") == 0)
+		    prev_dir = mallocstrcpy(NULL,
+			striponedir(filelist[selected]));
 
 		dir = opendir(filelist[selected]);
 		if (dir == NULL) {
@@ -331,34 +356,26 @@ char *do_browser(char *path, DIR *dir)
 		path = mallocstrcpy(path, filelist[selected]);
 
 		/* Start over again with the new path value. */
-		free_chararray(filelist, filelist_len);
 		goto change_browser_directory;
-	    /* Abort the browser. */
+	    /* Abort the file browser. */
 	    case NANO_EXIT_KEY:
 		abort = TRUE;
 		break;
 	}
+    }
 
-	if (abort)
-	    break;
-
-	browser_refresh();
-
-	kbinput = get_kbinput(edit, &meta_key, &func_key);
-	parse_browser_input(&kbinput, &meta_key, &func_key);
-    } while (kbinput != NANO_EXIT_KEY);
-
-    blank_edit();
     titlebar(NULL);
     edit_refresh();
     curs_set(1);
     if (old_const_update)
 	SET(CONST_UPDATE);
 
-    /* Clean up. */
     free(path);
     free(ans);
+
     free_chararray(filelist, filelist_len);
+    filelist = NULL;
+    filelist_len = 0;
 
     return retval;
 }
@@ -384,7 +401,8 @@ char *do_browse_from(const char *inpath)
      * at all.  If so, we'll just pass the current directory to
      * do_browser(). */
     if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
-	striponedir(path);
+	path = mallocstrassn(path, striponedir(path));
+
 	if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
 	    free(path);
 
@@ -399,19 +417,18 @@ char *do_browse_from(const char *inpath)
 #ifndef DISABLE_OPERATINGDIR
     /* If the resulting path isn't in the operating directory, use
      * the operating directory instead. */
-    if (check_operating_dir(path, FALSE)) {
-	if (path != NULL)
-	    free(path);
-	path = mallocstrcpy(NULL, operating_dir);
-    }
+    if (check_operating_dir(path, FALSE))
+	path = mallocstrcpy(path, operating_dir);
 #endif
 
     if (path != NULL)
 	dir = opendir(path);
 
+    /* If we can't open the path, get out. */
     if (dir == NULL) {
+	if (path != NULL)
+	    free(path);
 	beep();
-	free(path);
 	return NULL;
     }
 
@@ -419,16 +436,28 @@ char *do_browse_from(const char *inpath)
 }
 
 /* Set filelist to the list of files contained in the directory path,
- * set filelist_len to the number of files in that list, and set longest
- * to the width in columns of the longest filename in that list, up to
- * COLS - 1 (but at least 7).  Assume path exists and is a directory. */
+ * set filelist_len to the number of files in that list, set longest to
+ * the width in columns of the longest filename in that list (between 15
+ * and COLS), and set width to the number of files that we can display
+ * per line.  longest needs to be at least 15 columns in order to
+ * display ".. (parent dir)", as Pico does.  Assume path exists and is a
+ * directory. */
 void browser_init(const char *path, DIR *dir)
 {
     const struct dirent *nextdir;
-    size_t i = 0, path_len;
+    size_t i = 0, path_len = strlen(path);
+    int col = 0;
+	/* The maximum number of columns that the filenames will take
+	 * up. */
+    int line = 0;
+	/* The maximum number of lines that the filenames will take
+	 * up. */
+    int filesperline = 0;
+	/* The number of files that we can display per line. */
 
-    assert(dir != NULL);
+    assert(path != NULL && path[strlen(path) - 1] == '/' && dir != NULL);
 
+    /* Set longest to zero, just before we initialize it. */
     longest = 0;
 
     while ((nextdir = readdir(dir)) != NULL) {
@@ -437,20 +466,26 @@ void browser_init(const char *path, DIR *dir)
 	/* Don't show the "." entry. */
 	if (strcmp(nextdir->d_name, ".") == 0)
 	   continue;
-	i++;
 
 	d_len = strlenpt(nextdir->d_name);
 	if (d_len > longest)
-	    longest = (d_len > COLS - 1) ? COLS - 1 : d_len;
+	    longest = (d_len > COLS) ? COLS : d_len;
+
+	i++;
     }
 
-    filelist_len = i;
     rewinddir(dir);
+
+    /* Put 10 columns' worth of blank space between columns of filenames
+     * in the list whenever possible, as Pico does. */
     longest += 10;
 
-    filelist = (char **)nmalloc(filelist_len * sizeof(char *));
+    if (filelist != NULL)
+	free_chararray(filelist, filelist_len);
 
-    path_len = strlen(path);
+    filelist_len = i;
+
+    filelist = (char **)nmalloc(filelist_len * sizeof(char *));
 
     i = 0;
 
@@ -461,6 +496,7 @@ void browser_init(const char *path, DIR *dir)
 
 	filelist[i] = charalloc(path_len + strlen(nextdir->d_name) + 1);
 	sprintf(filelist[i], "%s%s", path, nextdir->d_name);
+
 	i++;
     }
 
@@ -468,12 +504,44 @@ void browser_init(const char *path, DIR *dir)
      * first time we scanned and the second.  i is the actual length of
      * filelist, so record it. */
     filelist_len = i;
+
     closedir(dir);
 
-    if (longest > COLS - 1)
-	longest = COLS - 1;
-    if (longest < 7)
-	longest = 7;
+    /* Make sure longest is between 15 and COLS. */
+    if (longest < 15)
+	longest = 15;
+    if (longest > COLS)
+	longest = COLS;
+
+    /* Set width to zero, just before we initialize it. */
+    width = 0;
+
+    for (i = 0; i < filelist_len && line < editwinrows; i++) {
+	/* Calculate the number of columns one filename will take up. */
+	col += longest;
+	filesperline++;
+
+	/* Add some space between the columns. */
+	col += 2;
+
+	/* If the next entry isn't going to fit on the current line,
+	 * move to the next line. */
+	if (col > COLS - longest) {
+	    line++;
+	    col = 0;
+
+	    /* If width isn't initialized yet, and we've taken up more
+	     * than one line, it means that width is equal to
+	     * filesperline. */
+	    if (width == 0)
+		width = filesperline;
+	}
+    }
+
+    /* If width isn't initialized yet, and we've taken up only one line,
+     * it means that width is equal to (COLS % longest). */
+    if (width == 0)
+	width = COLS % longest;
 }
 
 /* Determine the shortcut key corresponding to the values of kbinput
@@ -486,7 +554,7 @@ void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key)
     get_shortcut(browser_list, kbinput, meta_key, func_key);
 
     /* Pico compatibility. */
-    if (*meta_key == FALSE) {
+    if (!*meta_key) {
 	switch (*kbinput) {
 	    case ' ':
 		*kbinput = NANO_NEXTPAGE_KEY;
@@ -519,69 +587,136 @@ void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key)
     }
 }
 
-/* Calculate the number of columns needed to display the list of files
- * in the array filelist, if necessary, and then display the list of
- * files. */
+/* Set width to the number of files that we can display per line, if
+ * necessary, and display the list of files. */
 void browser_refresh(void)
 {
-    struct stat st;
-    size_t i = 0;
-    int col = 0, line = 0, filecols = 0;
-    size_t foo_len = mb_cur_max() * 7;
-    char *foo = charalloc(foo_len + 1);
+    static int uimax_digits = -1;
+    size_t i;
+    int col = 0;
+	/* The maximum number of columns that the filenames will take
+	 * up. */
+    int line = 0;
+	/* The maximum number of lines that the filenames will take
+	 * up. */
+    char *foo;
+	/* The file information that we'll display. */
 
-    if (width != 0)
-	i = width * editwinrows * ((selected / width) / editwinrows);
+    if (uimax_digits == -1)
+	uimax_digits = digits(UINT_MAX);
 
     blank_edit();
 
     wmove(edit, 0, 0);
 
-    for (; i < filelist_len && line <= editwinrows - 1; i++) {
-	char *disp = display_string(tail(filelist[i]), 0, longest,
-		FALSE);
+    i = width * editwinrows * ((selected / width) / editwinrows);
 
-	/* Highlight the currently selected file or directory. */
+    for (; i < filelist_len && line < editwinrows; i++) {
+	struct stat st;
+	const char *filetail = tail(filelist[i]);
+		/* The filename we display, minus the path. */
+	size_t filetaillen = strlenpt(filetail);
+		/* The length of the filename in columns. */
+	size_t foolen;
+		/* The length of the file information in columns. */
+	int foomaxlen = 7;
+		/* The maximum length of the file information in
+		 * columns: 7 for "--", "(dir)", or the file size, and
+		 * 12 for "(parent dir)". */
+	bool dots = (COLS >= 15 && filetaillen >= longest -
+		foomaxlen - 1);
+		/* Do we put an ellipsis before the filename?  Don't set
+		 * this to TRUE if we have fewer than 15 columns (i.e. 1
+		 * column for padding, plus 7 columns for a filename
+		 * other than ".."). */
+	char *disp = display_string(filetail, dots ? filetaillen -
+		longest + foomaxlen + 4 : 0, longest, FALSE);
+		/* If we put an ellipsis before the filename, reserve 1
+		 * column for padding, plus 7 columns for "--", "(dir)",
+		 * or the file size, plus 3 columns for the ellipsis. */
+
+	/* Start highlighting the currently selected file or
+	 * directory. */
 	if (i == selected)
 	    wattron(edit, reverse_attr);
 
 	blank_line(edit, line, col, longest);
-	mvwaddstr(edit, line, col, disp);
+
+	/* If dots is TRUE, we will display something like
+	 * "...ename". */
+	if (dots)
+	    mvwaddstr(edit, line, col, "...");
+	mvwaddstr(edit, line, dots ? col + 3 : col, disp);
+
 	free(disp);
 
 	col += longest;
-	filecols++;
 
-	/* Show file info also.  We don't want to report file sizes for
-	 * links, so we use lstat().  Also, stat() and lstat() return an
-	 * error if, for example, the file is deleted while the file
-	 * browser is open.  In that case, we report "--" as the file
-	 * info. */
+	/* Show information about the file.  We don't want to report
+	 * file sizes for links, so we use lstat(). */
 	if (lstat(filelist[i], &st) == -1 || S_ISLNK(st.st_mode)) {
-	    /* Aha!  It's a symlink!  Now, is it a dir?  If so, mark it
-	     * as such. */
-	    if (stat(filelist[i], &st) == 0 && S_ISDIR(st.st_mode)) {
-		strncpy(foo, _("(dir)"), foo_len);
-		foo[foo_len] = '\0';
-	    } else
-		strcpy(foo, "--");
+	    /* If the file doesn't exist (i.e. it's been deleted while
+	     * the file browser is open), or it's a symlink that doesn't
+	     * point to a directory, display "--". */
+	    if (stat(filelist[i], &st) == -1 || !S_ISDIR(st.st_mode))
+		foo = mallocstrcpy(NULL, "--");
+	    /* If the file is a symlink that points to a directory,
+	     * display it as a directory. */
+	    else
+		/* TRANSLATORS: Try to keep this at most 7
+		 * characters. */
+		foo = mallocstrcpy(NULL, _("(dir)"));
 	} else if (S_ISDIR(st.st_mode)) {
-	    strncpy(foo, _("(dir)"), foo_len);
-	    foo[foo_len] = '\0';
-	} else if (st.st_size < (1 << 10)) /* less than 1 k. */
-	    sprintf(foo, "%4u  B", (unsigned int)st.st_size);
-	else if (st.st_size < (1 << 20)) /* less than 1 meg. */
-	    sprintf(foo, "%4u KB", (unsigned int)(st.st_size >> 10));
-	else if (st.st_size < (1 << 30)) /* less than 1 gig. */
-	    sprintf(foo, "%4u MB", (unsigned int)(st.st_size >> 20));
-	else
-	    sprintf(foo, "%4u GB", (unsigned int)(st.st_size >> 30));
+	    /* If the file is a directory, display it as such. */
+	    if (strcmp(filetail, "..") == 0) {
+		/* TRANSLATORS: Try to keep this at most 12
+		 * characters. */
+		foo = mallocstrcpy(NULL, _("(parent dir)"));
+		foomaxlen = 12;
+	    } else
+		foo = mallocstrcpy(NULL, _("(dir)"));
+	} else {
+	    unsigned long result = st.st_size;
+	    char modifier;
 
-	mvwaddnstr(edit, line, col - strlen(foo), foo, foo_len);
+	    foo = charalloc(uimax_digits + 4);
 
+	    /* Bytes. */
+	    if (st.st_size < (1 << 10))
+		modifier = ' ';
+	    /* Kilobytes. */
+	    else if (st.st_size < (1 << 20)) {
+		result >>= 10;
+		modifier = 'K';
+	    /* Megabytes. */
+	    } else if (st.st_size < (1 << 30)) {
+		result >>= 20;
+		modifier = 'M';
+	    /* Gigabytes. */
+	    } else {
+		result >>= 30;
+		modifier = 'G';
+	    }
+
+	    sprintf(foo, "%4lu %cB", result, modifier);
+	}
+
+	/* Make sure foo takes up no more than foomaxlen columns. */
+	foolen = strlenpt(foo);
+	if (foolen > foomaxlen) {
+	    null_at(&foo, actual_x(foo, foomaxlen));
+	    foolen = foomaxlen;
+	}
+
+	mvwaddstr(edit, line, col - foolen, foo);
+
+	/* Finish highlighting the currently selected file or
+	 * directory. */
 	if (i == selected)
 	    wattroff(edit, reverse_attr);
 
+	free(foo);
+
 	/* Add some space between the columns. */
 	col += 2;
 
@@ -590,21 +725,36 @@ void browser_refresh(void)
 	if (col > COLS - longest) {
 	    line++;
 	    col = 0;
-
-	    /* Set the number of columns to display the list in, if
-	     * necessary, so that we don't divide by zero. */
-	    if (width == 0)
-		width = filecols;
 	}
 
 	wmove(edit, line, col);
     }
 
-    free(foo);
-
     wnoutrefresh(edit);
 }
 
+/* Look for needle.  If we find it, set selected to its location.  Note
+ * that needle must be an exact match for a file in the list.  The
+ * return value specifies whether we found anything. */
+bool browser_select_filename(const char *needle)
+{
+    size_t currselected;
+    bool found = FALSE;
+
+    for (currselected = 0; currselected < filelist_len;
+	currselected++) {
+	if (strcmp(filelist[currselected], needle) == 0) {
+	    found = TRUE;
+	    break;
+	}
+    }
+
+    if (found)
+	selected = currselected;
+
+    return found;
+}
+
 /* Set up the system variables for a filename search.  Return -1 if the
  * search should be canceled (due to Cancel, a blank search string, or a
  * failed regcomp()), return 0 on success, and return 1 on rerun calling
@@ -725,17 +875,18 @@ bool findnextfile(bool no_sameline, size_t begin, const char *needle)
     size_t currselected = selected;
 	/* The location in the current file list of the match we
 	 * find. */
-    const char *rev_start = tail(filelist[currselected]), *found = NULL;
+    const char *filetail = tail(filelist[currselected]);
+	/* The filename we display, minus the path. */
+    const char *rev_start = filetail, *found = NULL;
 
 #ifndef NANO_TINY
     if (ISSET(BACKWARDS_SEARCH))
-	rev_start += strlen(tail(filelist[currselected]));
+	rev_start += strlen(rev_start);
 #endif
 
     /* Look for needle in the current filename we're searching. */
     while (TRUE) {
-	found = strstrwrapper(tail(filelist[currselected]), needle,
-		rev_start);
+	found = strstrwrapper(filetail, needle, rev_start);
 
 	/* We've found a potential match.  If we're not allowed to find
 	 * a match on the same filename we started on and this potential
@@ -775,10 +926,12 @@ bool findnextfile(bool no_sameline, size_t begin, const char *needle)
 	if (currselected == begin)
 	    search_last_file = TRUE;
 
-	rev_start = tail(filelist[currselected]);
+	filetail = tail(filelist[currselected]);
+
+	rev_start = filetail;
 #ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH))
-	    rev_start += strlen(tail(filelist[currselected]));
+	    rev_start += strlen(rev_start);
 #endif
     }
 
@@ -893,27 +1046,35 @@ void do_fileresearch(void)
     filesearch_abort();
 }
 
+/* Select the first file in the list. */
 void do_first_file(void)
 {
     selected = 0;
 }
 
+/* Select the last file in the list. */
 void do_last_file(void)
 {
     selected = filelist_len - 1;
 }
 
-/* Strip one directory from the end of path. */
-void striponedir(char *path)
+/* Strip one directory from the end of path, and return the stripped
+ * path.  The returned string is dynamically allocated, and should be
+ * freed. */
+char *striponedir(const char *path)
 {
-    char *tmp;
+    char *retval, *tmp;
 
     assert(path != NULL);
 
-    tmp = strrchr(path, '/');
+    retval = mallocstrcpy(NULL, path);
+
+    tmp = strrchr(retval, '/');
 
     if (tmp != NULL)
- 	*tmp = '\0';
+ 	null_at(&retval, tmp - retval);
+
+    return retval;
 }
 
 #endif /* !DISABLE_BROWSER */
