@@ -1,4 +1,4 @@
-/* $Id: chars.c,v 1.115 2006/05/24 17:35:59 dolorous Exp $ */
+/* $Id: chars.c,v 1.119 2006/10/08 15:21:23 dolorous Exp $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
@@ -41,7 +41,7 @@ static const wchar_t bad_wchar = 0xFFFD;
 	 * Unicode FFFD (Replacement Character), unless we're
 	 * determining if it's a control character or searching for a
 	 * match to it. */
-static const char *bad_mbchar = "\xEF\xBF\xBD";
+static const char *const bad_mbchar = "\xEF\xBF\xBD";
 static const int bad_mbchar_len = 3;
 
 /* Enable UTF-8 support. */
@@ -514,10 +514,7 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
 	    break;
     }
 
-    if (n > 0)
-	return tolower(*s1) - tolower(*s2);
-    else
-	return 0;
+    return (n > 0) ? tolower(*s1) - tolower(*s2) : 0;
 }
 #endif
 
@@ -527,12 +524,14 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *s1_mb = charalloc(MB_CUR_MAX);
-	char *s2_mb = charalloc(MB_CUR_MAX);
+	char *s1_mb, *s2_mb;
 	wchar_t ws1, ws2;
 
 	assert(s1 != NULL && s2 != NULL);
 
+	s1_mb = charalloc(MB_CUR_MAX);
+	s2_mb = charalloc(MB_CUR_MAX);
+
 	while (n > 0 && *s1 != '\0' && *s2 != '\0') {
 	    bool bad_s1_mb = FALSE, bad_s2_mb = FALSE;
 	    int s1_mb_len, s2_mb_len;
@@ -620,13 +619,15 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *r_mb = charalloc(MB_CUR_MAX);
-	char *q_mb = charalloc(MB_CUR_MAX);
+	char *r_mb, *q_mb;
 	wchar_t wr, wq;
 	bool found_needle = FALSE;
 
 	assert(haystack != NULL && needle != NULL);
 
+	r_mb = charalloc(MB_CUR_MAX);
+	q_mb = charalloc(MB_CUR_MAX);
+
 	while (*haystack != '\0') {
 	    const char *r = haystack, *q = needle;
 	    int r_mb_len, q_mb_len;
@@ -726,13 +727,15 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *r_mb = charalloc(MB_CUR_MAX);
-	char *q_mb = charalloc(MB_CUR_MAX);
+	char *r_mb, *q_mb;
 	wchar_t wr, wq;
 	bool begin_line = FALSE, found_needle = FALSE;
 
 	assert(haystack != NULL && needle != NULL && rev_start != NULL);
 
+	r_mb = charalloc(MB_CUR_MAX);
+	q_mb = charalloc(MB_CUR_MAX);
+
 	while (!begin_line) {
 	    const char *r = rev_start, *q = needle;
 	    int r_mb_len, q_mb_len;
@@ -1020,8 +1023,7 @@ bool is_valid_mbstring(const char *s)
 
     return 
 #ifdef ENABLE_UTF8
-	use_utf8 ?
-	(mbstowcs(NULL, s, 0) != (size_t)-1) :
+	use_utf8 ? (mbstowcs(NULL, s, 0) != (size_t)-1) :
 #endif
 	TRUE;
 }
