@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.56 2006/06/18 15:26:51 dolorous Exp $ */
+/* $Id: cut.c,v 1.59 2006/10/18 20:27:52 dolorous Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -90,7 +90,7 @@ void cut_to_eol(void)
     else if (openfile->current != openfile->filebot) {
 	/* If we're at the end of the line, and it isn't the last line
 	 * of the file, move all the text from the current position up
-	 * to the beginning of the next line, i.e, the newline at the
+	 * to the beginning of the next line, i.e. the newline at the
 	 * end, into the cutbuffer. */
 	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current->next, 0);
@@ -194,6 +194,10 @@ void do_cut_text(
 		cb_save->data -= cb_save_len;
 	    } else
 		copy_from_filestruct(cutbuffer, cutbottom);
+
+	    /* Set the current place we want to where the text from the
+	     * cutbuffer ends. */
+	    openfile->placewewant = xplustabs();
 	}
 
 	/* Set NO_NEWLINES back to what it was before, since we're done
