@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.294 2006/06/14 13:19:43 dolorous Exp $ */
+/* $Id: files.c,v 1.310 2006/10/13 16:35:57 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -309,13 +309,13 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
 	fileptr->data[buf_len - 1] = '\0';
 #endif
 
-    if (*first_line_ins == TRUE) {
+    if (*first_line_ins) {
 	/* Special case: We're inserting with the cursor on the first
 	 * line. */
 	fileptr->prev = NULL;
 	fileptr->next = openfile->fileage;
 	fileptr->lineno = 1;
-	if (*first_line_ins == TRUE) {
+	if (*first_line_ins) {
 	    *first_line_ins = FALSE;
 	    /* If we're inserting into the first line of the file, then
 	     * we want to make sure that our edit buffer stays on the
@@ -540,7 +540,7 @@ void read_file(FILE *f, const char *filename)
 	openfile->filebot);
 
     /* If the NO_NEWLINES flag isn't set, and text has been added to
-     * the magicline (i.e, a file that doesn't end in a newline has been
+     * the magicline (i.e. a file that doesn't end in a newline has been
      * inserted at the end of the current buffer), add a new magicline,
      * and move the current line down to it. */
     if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0') {
@@ -584,8 +584,8 @@ void read_file(FILE *f, const char *filename)
  * reading with a 0 return value.  *f is set to the opened file. */
 int open_file(const char *filename, bool newfie, FILE **f)
 {
-    int fd;
     struct stat fileinfo;
+    int fd;
 
     assert(filename != NULL && f != NULL);
 
@@ -683,6 +683,7 @@ void do_insertfile(
     char *ans = mallocstrcpy(NULL, "");
 	/* The last answer the user typed on the statusbar. */
     filestruct *edittop_save = openfile->edittop;
+    size_t current_x_save = openfile->current_x;
     ssize_t current_y_save = openfile->current_y;
     bool at_edittop = FALSE;
 	/* Whether we're at the top of the edit window. */
@@ -765,13 +766,11 @@ void do_insertfile(
 		if (tmp == NULL)
 		    continue;
 
+		/* We have a file now.  Indicate this. */
 		free(answer);
 		answer = tmp;
 
-		/* We have a file now.  Indicate this and get out of the
-		 * statusbar prompt cleanly. */
 		i = 0;
-		do_prompt_abort();
 	    }
 #endif
 
@@ -811,8 +810,18 @@ void do_insertfile(
 
 		/* Save the command's output in the current buffer. */
 		execute_command(answer);
-	    } else {
+
+#ifdef ENABLE_MULTIBUFFER
+		if (ISSET(MULTIBUFFER)) {
+		    /* Move back to the beginning of the first line of
+		     * the buffer. */
+		    openfile->current = openfile->fileage;
+		    openfile->current_x = 0;
+		    openfile->placewewant = 0;
+		}
 #endif
+	    } else {
+#endif /* !NANO_TINY */
 		/* Make sure the path to the file specified in answer is
 		 * tilde-expanded. */
 		answer = mallocstrassn(answer,
@@ -835,20 +844,26 @@ void do_insertfile(
 	    {
 		filestruct *top_save = openfile->fileage;
 
-		/* If we didn't insert into a new buffer, and we were at
-		 * the top of the edit window before, set the saved
-		 * value of edittop to the new top of the edit window,
-		 * and update the current y-coordinate to account for
-		 * the number of lines inserted. */
+		/* If we were at the top of the edit window before, set
+		 * the saved value of edittop to the new top of the edit
+		 * window. */
 		if (at_edittop)
 		    edittop_save = openfile->fileage;
+
+		/* Update the current x-coordinate to account for the
+		 * number of characters inserted on the current line. */
+		openfile->current_x = strlen(openfile->filebot->data);
+		if (openfile->fileage == openfile->filebot)
+		    openfile->current_x += current_x_save;
+
+		/* Update the current y-coordinate to account for the
+		 * number of lines inserted. */
 		openfile->current_y += current_y_save;
 
-		/* If we didn't insert into a new buffer, unpartition
-		 * the filestruct so that it contains all the text
-		 * again.  Note that we've replaced the non-text
-		 * originally in the partition with the text in the
-		 * inserted file/executed command output. */
+		/* Unpartition the filestruct so that it contains all
+		 * the text again.  Note that we've replaced the
+		 * non-text originally in the partition with the text in
+		 * the inserted file/executed command output. */
 		unpartition_filestruct(&filepart);
 
 		/* Renumber starting with the beginning line of the old
@@ -882,7 +897,7 @@ void do_insertfile_void(void)
 {
 #ifdef ENABLE_MULTIBUFFER
     if (ISSET(VIEW_MODE) && !ISSET(MULTIBUFFER))
-	statusbar(_("Key illegal in non-multibuffer mode"));
+	statusbar(_("Key invalid in non-multibuffer mode"));
     else
 #endif
 	do_insertfile(
@@ -902,21 +917,29 @@ void do_insertfile_void(void)
  * able to go there. */
 char *get_full_path(const char *origpath)
 {
-    char *d_here, *d_there = NULL;
+    struct stat fileinfo;
+    char *d_here, *d_there, *d_there_file = NULL;
+    const char *last_slash;
+    bool path_only;
 
     if (origpath == NULL)
     	return NULL;
 
-    /* Get the current directory. */
+    /* Get the current directory.  If it doesn't exist, back up and try
+     * again until we get a directory that does, and use that as the
+     * current directory. */
     d_here = charalloc(PATH_MAX + 1);
     d_here = getcwd(d_here, PATH_MAX + 1);
 
-    if (d_here != NULL) {
-	const char *last_slash;
-	char *d_there_file = NULL;
-	bool path_only;
-	struct stat fileinfo;
+    while (d_here == NULL) {
+	if (chdir("..") == -1)
+	    break;
 
+	d_here = getcwd(d_here, PATH_MAX + 1);
+    }
+
+    /* If we succeeded, canonicalize it in d_here. */
+    if (d_here != NULL) {
 	align(&d_here);
 
 	/* If the current directory isn't "/", tack a slash onto the end
@@ -925,89 +948,91 @@ char *get_full_path(const char *origpath)
 	    d_here = charealloc(d_here, strlen(d_here) + 2);
 	    strcat(d_here, "/");
 	}
+    /* Otherwise, set d_here to "". */
+    } else
+	d_here = mallocstrcpy(NULL, "");
 
-	d_there = real_dir_from_tilde(origpath);
+    d_there = real_dir_from_tilde(origpath);
 
-	assert(d_there != NULL);
+    assert(d_there != NULL);
 
-	/* Stat d_there.  If stat() fails, assume that d_there refers to
-	 * a new file that hasn't been saved to disk yet.  Set path_only
-	 * to TRUE if d_there refers to a directory, and FALSE if
-	 * d_there refers to a file. */
-	path_only = !stat(d_there, &fileinfo) &&
-		S_ISDIR(fileinfo.st_mode);
+    /* If stat()ing d_there fails, assume that d_there refers to a new
+     * file that hasn't been saved to disk yet.  Set path_only to TRUE
+     * if d_there refers to a directory, and FALSE otherwise. */
+    path_only = stat(d_there, &fileinfo) == 0 &&
+	S_ISDIR(fileinfo.st_mode);
 
-	/* If path_only is TRUE, make sure d_there ends in a slash. */
-	if (path_only) {
-	    size_t d_there_len = strlen(d_there);
+    /* If path_only is TRUE, make sure d_there ends in a slash. */
+    if (path_only) {
+	size_t d_there_len = strlen(d_there);
 
-	    if (d_there[d_there_len - 1] != '/') {
-		d_there = charealloc(d_there, d_there_len + 2);
-		strcat(d_there, "/");
-	    }
+	if (d_there[d_there_len - 1] != '/') {
+	    d_there = charealloc(d_there, d_there_len + 2);
+	    strcat(d_there, "/");
 	}
+    }
 
-	/* Search for the last slash in d_there. */
-	last_slash = strrchr(d_there, '/');
+    /* Search for the last slash in d_there. */
+    last_slash = strrchr(d_there, '/');
 
-	/* If we didn't find one, then make sure the answer is in the
-	 * format "d_here/d_there". */
-	if (last_slash == NULL) {
-	    assert(!path_only);
+    /* If we didn't find one, then make sure the answer is in the format
+     * "d_here/d_there". */
+    if (last_slash == NULL) {
+	assert(!path_only);
 
-	    d_there_file = d_there;
-	    d_there = d_here;
+	d_there_file = d_there;
+	d_there = d_here;
+    } else {
+	/* If path_only is FALSE, then save the filename portion of the
+	 * answer (everything after the last slash) in d_there_file. */
+	if (!path_only)
+	    d_there_file = mallocstrcpy(NULL, last_slash + 1);
+
+	/* And remove the filename portion of the answer from
+	 * d_there. */
+	null_at(&d_there, last_slash - d_there + 1);
+
+	/* Go to the path specified in d_there. */
+	if (chdir(d_there) == -1) {
+	    free(d_there);
+	    d_there = NULL;
 	} else {
-	    /* If path_only is FALSE, then save the filename portion of
-	     * the answer, everything after the last slash, in
-	     * d_there_file. */
-	    if (!path_only)
-		d_there_file = mallocstrcpy(NULL, last_slash + 1);
+	    free(d_there);
 
-	    /* And remove the filename portion of the answer from
-	     * d_there. */
-	    null_at(&d_there, last_slash - d_there + 1);
+	    /* Get the full path. */
+	    d_there = charalloc(PATH_MAX + 1);
+	    d_there = getcwd(d_there, PATH_MAX + 1);
 
-	    /* Go to the path specified in d_there. */
-	    if (chdir(d_there) == -1) {
-		free(d_there);
-		d_there = NULL;
-	    } else {
-		/* Get the full path and save it in d_there. */
-		free(d_there);
+	    /* If we succeeded, canonicalize it in d_there. */
+	    if (d_there != NULL) {
+		align(&d_there);
 
-		d_there = charalloc(PATH_MAX + 1);
-		d_there = getcwd(d_there, PATH_MAX + 1);
+		/* If the current directory isn't "/", tack a slash onto
+		 * the end of it. */
+		if (strcmp(d_there, "/") != 0) {
+		    d_there = charealloc(d_there, strlen(d_there) + 2);
+		    strcat(d_there, "/");
+		}
+	    } else
+		/* Otherwise, set path_only to TRUE, so that we clean up
+		 * correctly, free all allocated memory, and return
+		 * NULL. */
+		path_only = TRUE;
 
-		if (d_there != NULL) {
-		    align(&d_there);
-
-		    if (strcmp(d_there, "/") != 0) {
-			/* Make sure d_there ends in a slash. */
-			d_there = charealloc(d_there,
-				strlen(d_there) + 2);
-			strcat(d_there, "/");
-		    }
-		} else
-		    /* If we couldn't get the full path, set path_only
-		     * to TRUE so that we clean up correctly, free all
-		     * allocated memory, and return NULL. */
-		    path_only = TRUE;
-
-		/* Finally, go back to the path specified in d_here,
-		 * where we were before. */
-		chdir(d_here);
-	    }
+	    /* Finally, go back to the path specified in d_here,
+	     * where we were before.  We don't check for a chdir()
+	     * error, since we can do nothing then. */
+	    chdir(d_here);
 
 	    /* Free d_here, since we're done using it. */
 	    free(d_here);
 	}
 
-	/* At this point, if path_only is FALSE and d_there exists,
+	/* At this point, if path_only is FALSE and d_there isn't NULL,
 	 * d_there contains the path portion of the answer and
 	 * d_there_file contains the filename portion of the answer.  If
-	 * this is the case, tack d_there_file onto the end of
-	 * d_there, so that d_there contains the complete answer. */
+	 * this is the case, tack the latter onto the end of the former.
+	 * d_there will then contain the complete answer. */
 	if (!path_only && d_there != NULL) {
 	    d_there = charealloc(d_there, strlen(d_there) +
 		strlen(d_there_file) + 1);
@@ -1137,9 +1162,9 @@ bool check_operating_dir(const char *currpath, bool allow_tabcomp)
 
     fullpath = get_full_path(currpath);
 
-    /* fullpath == NULL means some directory in the path doesn't exist
-     * or is unreadable.  If allow_tabcomp is FALSE, then currpath is
-     * what the user typed somewhere.  We don't want to report a
+    /* If fullpath is NULL, it means some directory in the path doesn't
+     * exist or is unreadable.  If allow_tabcomp is FALSE, then currpath
+     * is what the user typed somewhere.  We don't want to report a
      * non-existent directory as being outside the operating directory,
      * so we return FALSE.  If allow_tabcomp is TRUE, then currpath
      * exists, but is not executable.  So we say it isn't in the
@@ -1798,12 +1823,9 @@ int do_writeout(bool exiting)
 		if (tmp == NULL)
 		    continue;
 
+		/* We have a file now.  Indicate this. */
 		free(answer);
 		answer = tmp;
-
-		/* We have a file now.  Get out of the statusbar prompt
-		 * cleanly. */
-		do_prompt_abort();
 	    } else
 #endif /* !DISABLE_BROWSER */
 #ifndef NANO_TINY
@@ -2217,22 +2239,22 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	free(match2_mb);
 
 	mzero = charalloc(lastslash_len + common_len + 1);
-	sprintf(mzero, "%.*s%.*s", lastslash_len, buf, common_len,
-		matches[0]);
+
+	strncpy(mzero, buf, lastslash_len);
+	strncpy(mzero + lastslash_len, matches[0], common_len);
 
 	common_len += lastslash_len;
+	mzero[common_len] = '\0';
 
 	assert(common_len >= *place);
 
 	if (num_matches == 1 && is_dir(mzero)) {
-	    mzero[common_len] = '/';
-	    common_len++;
+	    mzero[common_len++] = '/';
 
 	    assert(common_len > *place);
 	}
 
-	if (num_matches > 1 && (common_len != *place ||
-		*lastwastab == FALSE))
+	if (num_matches > 1 && (common_len != *place || !*lastwastab))
 	    beep();
 
 	/* If there is more of a match to display on the statusbar, show
@@ -2248,7 +2270,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 		*place + 1);
 	    strncpy(buf, mzero, common_len);
 	    *place = common_len;
-	} else if (*lastwastab == FALSE || num_matches < 2)
+	} else if (!*lastwastab || num_matches < 2)
 	    *lastwastab = TRUE;
 	else {
 	    int longest_name = 0, columns, editline = 0;
@@ -2273,7 +2295,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 
 	    assert(longest_name <= COLS - 1);
 
-	    /* Each column will be (longest_name + 2) columns wide, i.e,
+	    /* Each column will be (longest_name + 2) columns wide, i.e.
 	     * two spaces between columns, except that there will be
 	     * only one space after the last column. */
 	    columns = (COLS + 1) / (longest_name + 2);
@@ -2319,7 +2341,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 
     /* Only refresh the edit window if we don't have a list of filename
      * matches on it. */
-    if (*list == FALSE)
+    if (!*list)
 	refresh_func();
 
     /* Enable el cursor. */
@@ -2357,6 +2379,7 @@ char *histfilename(void)
 	strcpy(nanohist, homedir);
 	strcpy(nanohist + homelen, "/.nano_history");
     }
+
     return nanohist;
 }
 
