@@ -1,4 +1,4 @@
-/* $Id: help.c,v 1.33 2006/06/19 16:04:42 dolorous Exp $ */
+/* $Id: help.c,v 1.53 2006/10/02 21:25:41 dolorous Exp $ */
 /**************************************************************************
  *   help.c                                                               *
  *                                                                        *
@@ -32,25 +32,28 @@
 static char *help_text = NULL;
 	/* The text displayed in the help window. */
 
-/* Our dynamic, shortcut list-compliant help function.  refresh_func is
- * the function we will call to refresh the edit window.*/
+/* Our main help browser function.  refresh_func is the function we will
+ * call to refresh the edit window. */
 void do_help(void (*refresh_func)(void))
 {
+    int kbinput = ERR;
+    bool meta_key, func_key, old_no_help = ISSET(NO_HELP);
+    bool abort = FALSE;
+	/* Whether we should abort the help browser. */
     size_t line = 0;
 	/* The line number in help_text of the first displayed help
 	 * line.  This variable is zero-based. */
     size_t last_line = 0;
 	/* The line number in help_text of the last help line.  This
 	 * variable is zero-based. */
-    int kbinput = ERR;
-    bool meta_key, func_key;
-    bool old_no_help = ISSET(NO_HELP);
 #ifndef DISABLE_MOUSE
     const shortcut *oldshortcut = currshortcut;
-	/* We will set currshortcut to allow clicking on the help
-	 * screen's shortcut list. */
+	/* The current shortcut list. */
 #endif
     const char *ptr;
+	/* The current line of help text. */
+    size_t old_line = (size_t)-1;
+	/* The line we were on before the current line. */
 
     curs_set(0);
     blank_edit();
@@ -76,6 +79,7 @@ void do_help(void (*refresh_func)(void))
     }
 
     bottombars(help_list);
+    wnoutrefresh(bottomwin);
 
     /* Get the last line of the help text. */
     ptr = help_text;
@@ -88,23 +92,52 @@ void do_help(void (*refresh_func)(void))
     if (last_line > 0)
 	last_line--;
 
-    do {
+    while (!abort) {
 	size_t i;
-	    /* Generic loop variable. */
-	size_t old_line = line;
-	    /* We redisplay the help only if it moved. */
 
-	ptr = help_text;
+	/* Display the help text if we don't have a key, or if the help
+	 * text has moved. */
+	if (kbinput == ERR || line != old_line) {
+	    blank_edit();
+
+	    ptr = help_text;
+
+	    /* Calculate where in the text we should be, based on the
+	     * page. */
+	    for (i = 0; i < line; i++) {
+		ptr += help_line_len(ptr);
+		if (*ptr == '\n')
+		    ptr++;
+	    }
+
+	    for (i = 0; i < editwinrows && *ptr != '\0'; i++) {
+		size_t j = help_line_len(ptr);
+
+		mvwaddnstr(edit, i, 0, ptr, j);
+		ptr += j;
+		if (*ptr == '\n')
+		    ptr++;
+	    }
+	}
+
+	wnoutrefresh(edit);
+
+	old_line = line;
+
+	kbinput = get_kbinput(edit, &meta_key, &func_key);
+	parse_help_input(&kbinput, &meta_key, &func_key);
 
 	switch (kbinput) {
 #ifndef DISABLE_MOUSE
 	    case KEY_MOUSE:
 		{
 		    int mouse_x, mouse_y;
+
 		    get_mouseinput(&mouse_x, &mouse_y, TRUE);
 		}
 		break;
 #endif
+	    /* Redraw the screen. */
 	    case NANO_REFRESH_KEY:
 		total_redraw();
 		break;
@@ -115,7 +148,7 @@ void do_help(void (*refresh_func)(void))
 		    line = 0;
 		break;
 	    case NANO_NEXTPAGE_KEY:
-		if (line + (editwinrows - 2) <= last_line)
+		if (line + (editwinrows - 1) < last_line)
 		    line += editwinrows - 2;
 		break;
 	    case NANO_PREVLINE_KEY:
@@ -123,7 +156,7 @@ void do_help(void (*refresh_func)(void))
 		    line--;
 		break;
 	    case NANO_NEXTLINE_KEY:
-		if (line + editwinrows <= last_line)
+		if (line + (editwinrows - 1) < last_line)
 		    line++;
 		break;
 	    case NANO_FIRSTLINE_ALTKEY:
@@ -136,35 +169,12 @@ void do_help(void (*refresh_func)(void))
 			line = last_line - (editwinrows - 1);
 		}
 		break;
+	    /* Abort the help browser. */
+	    case NANO_EXIT_KEY:
+		abort = TRUE;
+		break;
 	}
-
-	if ((kbinput != ERR && line == old_line) || kbinput ==
-		NANO_REFRESH_KEY)
-	    goto skip_redisplay;
-
-	blank_edit();
-
-	/* Calculate where in the text we should be, based on the
-	 * page. */
-	for (i = 0; i < line; i++) {
-	    ptr += help_line_len(ptr);
-	    if (*ptr == '\n')
-		ptr++;
-	}
-
-	for (i = 0; i < editwinrows && *ptr != '\0'; i++) {
-	    size_t j = help_line_len(ptr);
-
-	    mvwaddnstr(edit, i, 0, ptr, j);
-	    ptr += j;
-	    if (*ptr == '\n')
-		ptr++;
-	}
-
-  skip_redisplay:
-	kbinput = get_kbinput(edit, &meta_key, &func_key);
-	parse_help_input(&kbinput, &meta_key, &func_key);
-    } while (kbinput != NANO_EXIT_KEY);
+    }
 
 #ifndef DISABLE_MOUSE
     currshortcut = oldshortcut;
@@ -345,7 +355,7 @@ void help_init(void)
 #endif /* !NANO_TINY */
     else {
 	/* Default to the main help list. */
-	htx[0] = N_(" nano help text\n\n "
+	htx[0] = N_("Main nano help text\n\n "
 		"The nano editor is designed to emulate the "
 		"functionality and ease-of-use of the UW Pico text "
 		"editor.  There are four main sections of the editor.  "
@@ -354,15 +364,16 @@ void help_init(void)
 		"has been modified.  Next is the main editor window "
 		"showing the file being edited.  The status line is "
 		"the third line from the bottom and shows important "
-		"messages.  The bottom two lines show the most "
-		"commonly used shortcuts in the editor.\n\n");
-	htx[1] = N_(" The notation for shortcuts is as follows: "
-		"Control-key sequences are notated with a caret (^) "
-		"symbol and can be entered either by using the Control "
-		"(Ctrl) key or pressing the Escape (Esc) key twice.  "
-		"Escape-key sequences are notated with the Meta (M) "
-		"symbol and can be entered using either the Esc, Alt, "
-		"or Meta key depending on your keyboard setup.  ");
+		"messages.  ");
+	htx[1] = N_("The bottom two lines show the most commonly used "
+		"shortcuts in the editor.\n\n The notation for "
+		"shortcuts is as follows: Control-key sequences are "
+		"notated with a caret (^) symbol and can be entered "
+		"either by using the Control (Ctrl) key or pressing "
+		"the Escape (Esc) key twice.  Escape-key sequences are "
+		"notated with the Meta (M-) symbol and can be entered "
+		"using either the Esc, Alt, or Meta key depending on "
+		"your keyboard setup.  ");
 	htx[2] = N_("Also, pressing Esc twice and then typing a "
 		"three-digit decimal number from 000 to 255 will enter "
 		"the character with the corresponding value.  The "
@@ -564,7 +575,7 @@ void parse_help_input(int *kbinput, bool *meta_key, bool *func_key)
 {
     get_shortcut(help_list, kbinput, meta_key, func_key);
 
-    if (*meta_key == FALSE) {
+    if (!*meta_key) {
 	switch (*kbinput) {
 	    /* For consistency with the file browser. */
 	    case ' ':
@@ -575,6 +586,8 @@ void parse_help_input(int *kbinput, bool *meta_key, bool *func_key)
 		break;
 	    /* Cancel is equivalent to Exit here. */
 	    case NANO_CANCEL_KEY:
+	    case 'E':
+	    case 'e':
 		*kbinput = NANO_EXIT_KEY;
 		break;
 	}
