@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.555 2006/06/25 17:25:13 dolorous Exp $ */
+/* $Id: nano.c,v 1.572 2006/10/26 15:55:39 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -29,15 +29,16 @@
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
-#include <sys/ioctl.h>
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
 #include <termios.h>
-
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
+#ifndef NANO_TINY
+#include <sys/ioctl.h>
+#endif
 
 #ifdef ENABLE_NANORC
 static bool no_rcfiles = FALSE;
@@ -48,8 +49,8 @@ static struct termios oldterm;
 static struct sigaction act;
 	/* For all our fun signal handlers. */
 
-/* Create a new filestruct node.  Note that we specifically do not set
- * prevnode->next equal to the new line. */
+/* Create a new filestruct node.  Note that we do not set prevnode->next
+ * to the new line. */
 filestruct *make_new_node(filestruct *prevnode)
 {
     filestruct *newnode = (filestruct *)nmalloc(sizeof(filestruct));
@@ -519,19 +520,19 @@ void free_openfilestruct(openfilestruct *src)
 /* Display a warning about a key disabled in view mode. */
 void print_view_warning(void)
 {
-    statusbar(_("Key illegal in VIEW mode"));
+    statusbar(_("Key invalid in view mode"));
 }
 
 /* What we do when we're all set to exit. */
 void finish(void)
 {
+    /* Blank the statusbar (and shortcut list, if applicable), and move
+     * the cursor to the last line of the screen. */
     if (!ISSET(NO_HELP))
 	blank_bottombars();
     else
 	blank_statusbar();
-
     wrefresh(bottomwin);
-
     endwin();
 
     /* Restore the old terminal settings. */
@@ -674,7 +675,7 @@ void disable_mouse_support(void)
 /* Enable mouse support. */
 void enable_mouse_support(void)
 {
-    mousemask(BUTTON1_RELEASED, NULL);
+    mousemask(ALL_MOUSE_EVENTS, NULL);
     mouseinterval(50);
 }
 
@@ -724,7 +725,7 @@ void print_opt_full(const char *shortflag
 /* Explain how to properly use nano and its command line options. */
 void usage(void)
 {
-    printf(_("Usage: nano [OPTIONS] [[+LINE[,COLUMN]] FILE]...\n\n"));
+    printf(_("Usage: nano [OPTIONS] [[+LINE,COLUMN] FILE]...\n\n"));
     printf(
 #ifdef HAVE_GETOPT_LONG
 	_("Option\t\tGNU long option\t\tMeaning\n")
@@ -733,7 +734,7 @@ void usage(void)
 #endif
 	);
     print_opt("-h, -?", "--help", N_("Show this message"));
-    print_opt(_("+LINE[,COLUMN]"), "",
+    print_opt(_("+LINE,COLUMN"), "",
 	N_("Start at line LINE, column COLUMN"));
 #ifndef NANO_TINY
     print_opt("-A", "--smarthome", N_("Enable smart home key"));
@@ -766,7 +767,7 @@ void usage(void)
     print_opt("-N", "--noconvert",
 	N_("Don't convert files from DOS/Mac format"));
 #endif
-    print_opt("-O", "--morespace", N_("Use more space for editing"));
+    print_opt("-O", "--morespace", N_("Use one more line for editing"));
 #ifndef DISABLE_JUSTIFY
     print_opt(_("-Q <str>"), _("--quotestr=<str>"),
 	N_("Quoting string"));
@@ -776,7 +777,7 @@ void usage(void)
     print_opt("-S", "--smooth", N_("Smooth scrolling"));
 #endif
     print_opt(_("-T <#cols>"), _("--tabsize=<#cols>"),
-	N_("Set width of a tab in cols to #cols"));
+	N_("Set width of a tab to #cols columns"));
 #ifndef NANO_TINY
     print_opt("-U", "--quickblank", N_("Do quick statusbar blanking"));
 #endif
@@ -788,7 +789,7 @@ void usage(void)
 #endif
 #ifdef ENABLE_COLOR
     print_opt(_("-Y <str>"), _("--syntax=<str>"),
-	N_("Syntax definition to use"));
+	N_("Syntax definition to use for coloring"));
 #endif
     print_opt("-c", "--const", N_("Constantly show cursor position"));
     print_opt("-d", "--rebinddelete",
@@ -811,7 +812,7 @@ void usage(void)
 	N_("Preserve XON (^Q) and XOFF (^S) keys"));
 #ifndef DISABLE_WRAPJUSTIFY
     print_opt(_("-r <#cols>"), _("--fill=<#cols>"),
-	N_("Set fill cols to (wrap lines at) #cols"));
+	N_("Set wrapping point at column #cols"));
 #endif
 #ifndef DISABLE_SPELLER
     print_opt(_("-s <prog>"), _("--speller=<prog>"),
@@ -819,7 +820,7 @@ void usage(void)
 #endif
     print_opt("-t", "--tempfile",
 	N_("Auto save on exit, don't prompt"));
-    print_opt("-v", "--view", N_("View (read only) mode"));
+    print_opt("-v", "--view", N_("View mode (read-only)"));
 #ifndef DISABLE_WRAPPING
     print_opt("-w", "--nowrap", N_("Don't wrap long lines"));
 #endif
@@ -871,7 +872,7 @@ void version(void)
 #ifdef DISABLE_WRAPPING
     printf(" --disable-wrapping");
 #endif
-#if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
+#ifdef DISABLE_ROOTWRAPPING
     printf(" --disable-wrapping-as-root");
 #endif
 #ifdef ENABLE_COLOR
@@ -1011,7 +1012,8 @@ RETSIGTYPE do_suspend(int signal)
     disable_mouse_support();
 #endif
 
-    /* Blank the screen, and move the cursor to the last line of it. */
+    /* Blank the screen, and move the cursor to the last line of the
+     * screen. */
     erase();
     move(LINES - 1, 0);
     refresh();
@@ -1056,6 +1058,7 @@ RETSIGTYPE do_continue(int signal)
 
     /* Redraw the contents of the windows that need it. */
     blank_statusbar();
+    wnoutrefresh(bottomwin);
     total_refresh();
 #endif
 }
@@ -1116,6 +1119,7 @@ RETSIGTYPE handle_sigwinch(int signal)
 
     /* Redraw the contents of the windows that need it. */
     blank_statusbar();
+    wnoutrefresh(bottomwin);
     currshortcut = main_list;
     total_refresh();
 
@@ -1322,7 +1326,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
     if (allow_funcs) {
 	/* If we got a mouse click and it was on a shortcut, read in the
 	 * shortcut character. */
-	if (*func_key == TRUE && input == KEY_MOUSE) {
+	if (*func_key && input == KEY_MOUSE) {
 	    if (do_mouse())
 		input = get_kbinput(edit, meta_key, func_key);
 	    else {
@@ -1360,9 +1364,8 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 
     /* If we got a non-high-bit control key, a meta key sequence, or a
      * function key, and it's not a shortcut or toggle, throw it out. */
-    if (*s_or_t == FALSE) {
-	if (is_ascii_cntrl_char(input) || *meta_key == TRUE ||
-		*func_key == TRUE) {
+    if (!*s_or_t) {
+	if (is_ascii_cntrl_char(input) || *meta_key || *func_key) {
 	    statusbar(_("Unknown Command"));
 	    beep();
 	    *meta_key = FALSE;
@@ -1376,7 +1379,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	 * it's a normal text character.  Display the warning if we're
 	 * in view mode, or add the character to the input buffer if
 	 * we're not. */
-	if (input != ERR && *s_or_t == FALSE) {
+	if (input != ERR && !*s_or_t) {
 	    if (ISSET(VIEW_MODE))
 		print_view_warning();
 	    else {
@@ -1392,13 +1395,13 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	 * output all the characters in the input buffer if it isn't
 	 * empty.  Note that it should be empty if we're in view
 	 * mode. */
-	 if (*s_or_t == TRUE || get_key_buffer_len() == 0) {
+	 if (*s_or_t || get_key_buffer_len() == 0) {
 #ifndef DISABLE_WRAPPING
 	    /* If we got a shortcut or toggle, and it's not the shortcut
 	     * for verbatim input, turn off prepending of wrapped
 	     * text. */
-	    if (*s_or_t == TRUE && (!have_shortcut || s == NULL ||
-		s->func != do_verbatim_input))
+	    if (*s_or_t && (!have_shortcut || s == NULL || s->func !=
+		do_verbatim_input))
 		wrap_reset();
 #endif
 
@@ -1726,7 +1729,7 @@ int main(int argc, char **argv)
 	if (locale != NULL && (strcasestr(locale, "UTF8") != NULL ||
 		strcasestr(locale, "UTF-8") != NULL)) {
 #ifdef USE_SLANG
-	    SLutf8_enable(TRUE);
+	    SLutf8_enable(1);
 #endif
 	    utf8_init();
 	}
@@ -1740,9 +1743,9 @@ int main(int argc, char **argv)
     textdomain(PACKAGE);
 #endif
 
-#if !defined(ENABLE_NANORC) && defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
-    /* If we don't have rcfile support, we're root, and
-     * --disable-wrapping-as-root is used, turn wrapping off. */
+#if !defined(ENABLE_NANORC) && defined(DISABLE_ROOTWRAPPING)
+    /* If we don't have rcfile support, --disable-wrapping-as-root is
+     * used, and we're root, turn wrapping off. */
     if (geteuid() == NANO_ROOT_UID)
 	SET(NO_WRAP);
 #endif
@@ -1919,7 +1922,7 @@ int main(int argc, char **argv)
 	}
     }
 
-    /* If the executable filename starts with 'r', we use restricted
+    /* If the executable filename starts with 'r', enable restricted
      * mode. */
     if (*(tail(argv[0])) == 'r')
 	SET(RESTRICTED);
@@ -2005,7 +2008,9 @@ int main(int argc, char **argv)
 	    tabsize = tabsize_cpy;
 	flags |= flags_cpy;
     }
-#if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
+#ifdef DISABLE_ROOTWRAPPING
+    /* If we don't have any rcfiles, --disable-wrapping-as-root is used,
+     * and we're root, turn wrapping off. */
     else if (geteuid() == NANO_ROOT_UID)
 	SET(NO_WRAP);
 #endif
@@ -2232,6 +2237,10 @@ int main(int argc, char **argv)
 	}
 #endif
 
+	/* Just in case we were at the statusbar prompt, make sure the
+	 * statusbar cursor position is reset. */
+	do_prompt_abort();
+
 	/* If constant cursor position display is on, and there are no
 	 * keys waiting in the input buffer, display the current cursor
 	 * position on the statusbar. */
