@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.173 2006/06/09 16:57:41 dolorous Exp $ */
+/* $Id: nano.h,v 1.183 2006/08/22 18:48:13 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -29,7 +29,7 @@
 #endif
 
 #ifdef __TANDEM
-/* Tandem NonStop Kernel. */
+/* Tandem NonStop Kernel support. */
 #include <floss.h>
 #define NANO_ROOT_UID 65535
 #else
@@ -138,14 +138,8 @@
 #define getline ngetline
 #endif
 
-/* If we don't have regex support, turn the color support off, as it
- * depends on the use of regexes. */
-#ifndef HAVE_REGEX_H
-#undef ENABLE_COLOR 
-#endif
-
-/* If we aren't using ncurses, turn the mouse support off, as it's
- * ncurses-specific. */
+/* If we aren't using ncurses with mouse support, turn the mouse support
+ * off, as it's useless then. */
 #ifndef NCURSES_MOUSE_VERSION
 #define DISABLE_MOUSE 1
 #endif
@@ -164,7 +158,7 @@ typedef enum {
 } append_type;
 
 typedef enum {
-    UP, DOWN
+    UP_DIR, DOWN_DIR
 } scroll_dir;
 
 typedef enum {
@@ -297,11 +291,12 @@ typedef struct shortcut {
 #ifndef DISABLE_HELP
     const char *help;
 	/* The help file entry text for this function. */
-#endif
     bool blank_after;
 	/* Whether there should be a blank line after the help entry
 	 * text for this function. */
-    /* Key values that aren't used should be set to NANO_NO_KEY. */
+#endif
+    /* Note: Key values that aren't used should be set to
+     * NANO_NO_KEY. */
     int ctrlval;
 	/* The special sentinel key or control key we want bound, if
 	 * any. */
@@ -326,9 +321,11 @@ typedef struct toggle {
    const char *desc;
 	/* The description of the toggle, e.g. "Cut to end"; we'll
 	 * append Enabled or Disabled to it. */
+#ifndef DISABLE_HELP
    bool blank_after;
 	/* Whether there should be a blank line after the description of
 	 * the toggle. */
+#endif
    long flag;
 	/* Which flag actually gets toggled. */
    struct toggle *next;
@@ -547,8 +544,8 @@ typedef struct rcoption {
 #define NANO_DELETE_KEY			NANO_CONTROL_D
 #define NANO_BACKSPACE_KEY		NANO_CONTROL_H
 #define NANO_TAB_KEY			NANO_CONTROL_I
-#define NANO_INDENTMARKED_KEY		NANO_ALT_RCURLYBRACKET
-#define NANO_UNINDENTMARKED_KEY		NANO_ALT_LCURLYBRACKET
+#define NANO_INDENT_KEY			NANO_ALT_RCURLYBRACKET
+#define NANO_UNINDENT_KEY		NANO_ALT_LCURLYBRACKET
 #define NANO_SUSPEND_KEY		NANO_CONTROL_Z
 #define NANO_ENTER_KEY			NANO_CONTROL_M
 #define NANO_TOFILES_KEY		NANO_CONTROL_T
@@ -566,7 +563,6 @@ typedef struct rcoption {
 #define NANO_SCROLLDOWN_KEY		NANO_ALT_PLUS
 #define NANO_SCROLLUP_ALTKEY		NANO_ALT_UNDERSCORE
 #define NANO_SCROLLDOWN_ALTKEY		NANO_ALT_EQUALS
-#define NANO_CUTTILLEND_KEY		NANO_CONTROL_X
 #define NANO_CUTTILLEND_ALTKEY		NANO_ALT_T
 #define NANO_PARABEGIN_KEY		NANO_CONTROL_W
 #define NANO_PARABEGIN_ALTKEY		NANO_ALT_LPARENTHESIS
