@@ -1,4 +1,4 @@
-/* $Id: prompt.c,v 1.60 2006/06/03 19:36:02 dolorous Exp $ */
+/* $Id: prompt.c,v 1.69 2006/09/29 15:45:10 dolorous Exp $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -33,6 +33,10 @@ static size_t statusbar_x = (size_t)-1;
 	/* The cursor position in answer. */
 static size_t statusbar_pww = (size_t)-1;
 	/* The place we want in answer. */
+static size_t old_statusbar_x = (size_t)-1;
+	/* The old cursor position in answer, if any. */
+static size_t old_pww = (size_t)-1;
+	/* The old place we want in answer, if any. */
 static bool reset_statusbar_x = FALSE;
 	/* Should we reset the cursor position at the statusbar
 	 * prompt? */
@@ -71,7 +75,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
     if (allow_funcs) {
 	/* If we got a mouse click and it was on a shortcut, read in the
 	 * shortcut character. */
-	if (*func_key == TRUE && input == KEY_MOUSE) {
+	if (*func_key && input == KEY_MOUSE) {
 	    if (do_statusbar_mouse())
 		input = get_kbinput(bottomwin, meta_key, func_key);
 	    else {
@@ -97,7 +101,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 #ifndef NANO_TINY
 	input == NANO_NEXTWORD_KEY ||
 #endif
-	(*meta_key == TRUE && (
+	(*meta_key && (
 #ifndef NANO_TINY
 	input == NANO_PREVWORD_KEY || input == NANO_BRACKET_KEY ||
 #endif
@@ -108,9 +112,8 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 
     /* If we got a non-high-bit control key, a meta key sequence, or a
      * function key, and it's not a shortcut or toggle, throw it out. */
-    if (*s_or_t == FALSE) {
-	if (is_ascii_cntrl_char(input) || *meta_key == TRUE ||
-		*func_key == TRUE) {
+    if (!*s_or_t) {
+	if (is_ascii_cntrl_char(input) || *meta_key || *func_key) {
 	    beep();
 	    *meta_key = FALSE;
 	    *func_key = FALSE;
@@ -123,7 +126,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	 * it's a normal text character.  Display the warning if we're
 	 * in view mode, or add the character to the input buffer if
 	 * we're not. */
-	if (input != ERR && *s_or_t == FALSE) {
+	if (input != ERR && !*s_or_t) {
 	    /* If we're using restricted mode, the filename isn't blank,
 	     * and we're at the "Write File" prompt, disable text
 	     * input. */
@@ -139,7 +142,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	/* If we got a shortcut, or if there aren't any other characters
 	 * waiting after the one we read in, we need to display all the
 	 * characters in the input buffer if it isn't empty. */
-	 if (*s_or_t == TRUE || get_key_buffer_len() == 0) {
+	 if (*s_or_t || get_key_buffer_len() == 0) {
 	    if (kbinput != NULL) {
 		/* Display all the characters in the input buffer at
 		 * once, filtering out control characters. */
@@ -192,7 +195,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 		    do_statusbar_next_word(FALSE);
 		    break;
 		case NANO_PREVWORD_KEY:
-		    if (*meta_key == TRUE)
+		    if (*meta_key)
 			do_statusbar_prev_word(FALSE);
 		    break;
 #endif
@@ -204,12 +207,12 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 		    break;
 #ifndef NANO_TINY
 		case NANO_BRACKET_KEY:
-		    if (*meta_key == TRUE)
+		    if (*meta_key)
 			do_statusbar_find_bracket();
 		    break;
 #endif
 		case NANO_VERBATIM_KEY:
-		    if (*meta_key == TRUE) {
+		    if (*meta_key) {
 			/* If we're using restricted mode, the filename
 			 * isn't blank, and we're at the "Write File"
 			 * prompt, disable verbatim input. */
@@ -221,10 +224,12 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 
 			    do_statusbar_verbatim_input(&got_enter);
 
-			    /* If we got the Enter key, set input to the
-			     * key value for Enter, and set finished to
-			     * TRUE to indicate that we're done. */
+			    /* If we got the Enter key, remove it from
+			     * the input buffer, set input to the key
+			     * value for Enter, and set finished to TRUE
+			     * to indicate that we're done. */
 			    if (got_enter) {
+				get_input(NULL, 1);
 				input = NANO_ENTER_KEY;
 				*finished = TRUE;
 			    }
@@ -840,7 +845,7 @@ void do_statusbar_find_bracket(void)
 }
 #endif /* !NANO_TINY */
 
-/* Return the placewewant associated with statusbar_x, i.e, the
+/* Return the placewewant associated with statusbar_x, i.e. the
  * zero-based column position of the cursor.  The value will be no
  * smaller than statusbar_x. */
 size_t statusbar_xplustabs(void)
@@ -902,16 +907,18 @@ void update_statusbar_line(const char *curranswer, size_t index)
     reset_statusbar_cursor();
 
     wattroff(bottomwin, reverse_attr);
+
+    wnoutrefresh(bottomwin);
 }
 
 /* Return TRUE if we need an update after moving horizontally, and FALSE
- * otherwise.  We need one if old_pww and statusbar_pww are on different
- * pages. */
-bool need_statusbar_horizontal_update(size_t old_pww)
+ * otherwise.  We need one if pww_save and statusbar_pww are on
+ * different pages. */
+bool need_statusbar_horizontal_update(size_t pww_save)
 {
     size_t start_col = strlenpt(prompt) + 1;
 
-    return get_statusbar_page_start(start_col, start_col + old_pww) !=
+    return get_statusbar_page_start(start_col, start_col + pww_save) !=
 	get_statusbar_page_start(start_col, start_col + statusbar_pww);
 }
 
@@ -964,13 +971,20 @@ int get_prompt_string(bool allow_tabs,
     answer = mallocstrcpy(answer, curranswer);
     curranswer_len = strlen(answer);
 
-    /* Only put statusbar_x at the end of the string (and change
-     * statusbar_pww to match) if it's uninitialized, if it would be
-     * past the end of curranswer, or if reset_statusbar_x is TRUE.
-     * Otherwise, leave it alone.  This is so the cursor position stays
-     * at the same place if a prompt-changing toggle is pressed. */
-    if (statusbar_x == (size_t)-1 || statusbar_x > curranswer_len ||
-		reset_statusbar_x) {
+    /* If reset_statusbar_x is TRUE, restore statusbar_x and
+     * statusbar_pww to what they were before this prompt.  Then, if
+     * statusbar_x is uninitialized or past the end of curranswer, put
+     * statusbar_x at the end of the string and update statusbar_pww
+     * based on it.  We do these things so that the cursor position
+     * stays at the right place if a prompt-changing toggle is pressed,
+     * or if this prompt was started from another prompt and we cancel
+     * out of it. */
+    if (reset_statusbar_x) {
+	statusbar_x = old_statusbar_x;
+	statusbar_pww = old_pww;
+    }
+
+    if (statusbar_x == (size_t)-1 || statusbar_x > curranswer_len) {
 	statusbar_x = curranswer_len;
 	statusbar_pww = statusbar_xplustabs();
     }
@@ -1121,13 +1135,22 @@ int get_prompt_string(bool allow_tabs,
     }
 #endif
 
-    /* We finished putting in an answer or ran a normal shortcut's
-     * associated function, so reset statusbar_x and statusbar_pww. */
+    /* We've finished putting in an answer or run a normal shortcut's
+     * associated function, so reset statusbar_x and statusbar_pww.  If
+     * we've finished putting in an answer, reset the statusbar cursor
+     * position too. */
     if (kbinput == NANO_CANCEL_KEY || kbinput == NANO_ENTER_KEY ||
 	ran_func) {
-	statusbar_x = (size_t)-1;
-	statusbar_pww = (size_t)-1;
-    }
+	statusbar_x = old_statusbar_x;
+	statusbar_pww = old_pww;
+
+	if (!ran_func)
+	    reset_statusbar_x = TRUE;
+    /* Otherwise, we're still putting in an answer or a shortcut with
+     * an associated function, so leave the statusbar cursor position
+     * alone. */
+    } else
+	reset_statusbar_x = FALSE;
 
     return kbinput;
 }
@@ -1191,16 +1214,19 @@ int do_prompt(bool allow_tabs,
     free(prompt);
     prompt = NULL;
 
-    reset_statusbar_x = FALSE;
+    /* We're done with the prompt, so save the statusbar cursor
+     * position. */
+    old_statusbar_x = statusbar_x;
+    old_pww = statusbar_pww;
 
+    /* If we left the prompt via Cancel or Enter, set the return value
+     * properly. */
     switch (retval) {
 	case NANO_CANCEL_KEY:
 	    retval = -1;
-	    reset_statusbar_x = TRUE;
 	    break;
 	case NANO_ENTER_KEY:
 	    retval = (answer[0] == '\0') ? -2 : 0;
-	    reset_statusbar_x = TRUE;
 	    break;
     }
 
@@ -1223,10 +1249,13 @@ int do_prompt(bool allow_tabs,
 }
 
 /* This function forces a reset of the statusbar cursor position.  It
- * should only be called after do_prompt(), and is only needed if the
- * user leaves the prompt via something other than Enter or Cancel. */
+ * should be called when we get out of all statusbar prompts. */
 void do_prompt_abort(void)
 {
+    /* Uninitialize the old cursor position in answer. */
+    old_statusbar_x = (size_t)-1;
+    old_pww = (size_t)-1;
+
     reset_statusbar_x = TRUE;
 }
 
@@ -1310,10 +1339,10 @@ int do_yesno_prompt(bool all, const char *msg)
 	    case KEY_MOUSE:
 		get_mouseinput(&mouse_x, &mouse_y, FALSE);
 
-		if (mouse_x != -1 && mouse_y != -1 && !ISSET(NO_HELP) &&
-			wenclose(bottomwin, mouse_y, mouse_x) &&
-			mouse_x < (width * 2) && mouse_y - (2 -
-			no_more_space()) - editwinrows - 1 >= 0) {
+		if (wenclose(bottomwin, mouse_y, mouse_x) &&
+			!ISSET(NO_HELP) && mouse_x < (width * 2) &&
+			mouse_y - (2 - no_more_space()) -
+			editwinrows - 1 >= 0) {
 		    int x = mouse_x / width;
 			/* Calculate the x-coordinate relative to the
 			 * two columns of the Yes/No/All shortcuts in
@@ -1322,7 +1351,7 @@ int do_yesno_prompt(bool all, const char *msg)
 			editwinrows - 1;
 			/* Calculate the y-coordinate relative to the
 			 * beginning of the Yes/No/All shortcuts in
-			 * bottomwin, i.e, with the sizes of topwin,
+			 * bottomwin, i.e. with the sizes of topwin,
 			 * edit, and the first line of bottomwin
 			 * subtracted out. */
 
