@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.350 2006/06/21 16:54:51 dolorous Exp $ */
+/* $Id: proto.h,v 1.365 2006/08/26 16:50:51 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -26,7 +26,7 @@
 
 #include "nano.h"
 
-/* Public externs.  See global.c for descriptions of them. */
+/* Public external variables.  See global.c for their descriptions. */
 #ifndef NANO_TINY
 extern sigjmp_buf jump_buf;
 extern bool jump_buf_main;
@@ -148,6 +148,7 @@ char *do_browse_from(const char *inpath);
 void browser_init(const char *path, DIR *dir);
 void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key);
 void browser_refresh(void);
+bool browser_select_filename(const char *needle);
 int filesearch_init(void);
 bool findnextfile(bool no_sameline, size_t begin, const char *needle);
 void findnextfile_wrap_reset(void);
@@ -156,7 +157,7 @@ void do_filesearch(void);
 void do_fileresearch(void);
 void do_first_file(void);
 void do_last_file(void);
-void striponedir(char *path);
+char *striponedir(const char *path);
 #endif
 
 /* Public functions in chars.c. */
@@ -341,16 +342,19 @@ void save_history(void);
 /* Public functions in global.c. */
 size_t length_of_list(const shortcut *s);
 #ifndef NANO_TINY
-void toggle_init_one(int val, const char *desc, bool blank_after, long
-	flag);
+void toggle_init_one(int val
+#ifndef DISABLE_HELP
+	, const char *desc, bool blank_after
+#endif
+	, long flag);
 void toggle_init(void);
 #endif
-void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
+void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc
 #ifndef DISABLE_HELP
-	const char *help,
+	, const char *help, bool blank_after
 #endif
-	bool blank_after, int metaval, int funcval, int miscval, bool
-	view, void (*func)(void));
+	, int metaval, int funcval, int miscval, bool view, void
+	(*func)(void));
 void shortcut_init(bool unjustify);
 void free_shortcutage(shortcut **shortcutage);
 #ifdef DEBUG
@@ -388,11 +392,25 @@ void do_prev_word_void(void);
 #endif
 void do_home(void);
 void do_end(void);
-void do_up(void);
+void do_up(
+#ifndef NANO_TINY
+	bool scroll_only
+#else
+	void
+#endif
+	);
+void do_up_void(void);
 #ifndef NANO_TINY
 void do_scroll_up(void);
 #endif
-void do_down(void);
+void do_down(
+#ifndef NANO_TINY
+	bool scroll_only
+#else
+	void
+#endif
+	);
+void do_down_void(void);
 #ifndef NANO_TINY
 void do_scroll_down(void);
 #endif
@@ -500,7 +518,7 @@ size_t statusbar_xplustabs(void);
 size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reset_statusbar_cursor(void);
 void update_statusbar_line(const char *curranswer, size_t index);
-bool need_statusbar_horizontal_update(size_t old_pww);
+bool need_statusbar_horizontal_update(size_t pww_save);
 void total_statusbar_refresh(void (*refresh_func)(void));
 int get_prompt_string(bool allow_tabs,
 #ifndef DISABLE_TABCOMP
@@ -612,9 +630,9 @@ void do_delete(void);
 void do_backspace(void);
 void do_tab(void);
 #ifndef NANO_TINY
-void do_indent_marked(ssize_t cols);
-void do_indent_marked_void(void);
-void do_unindent_marked_void(void);
+void do_indent(ssize_t cols);
+void do_indent_void(void);
+void do_unindent(void);
 #endif
 void do_enter(void);
 #ifndef NANO_TINY
@@ -717,10 +735,9 @@ void unget_kbinput(int kbinput, bool meta_key, bool func_key);
 int *get_input(WINDOW *win, size_t input_len);
 int get_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
 int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
-int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
-	*ignore_seq);
+int get_escape_seq_kbinput(const int *seq, size_t seq_len);
 int get_escape_seq_abcd(int kbinput);
-int parse_escape_seq_kbinput(int kbinput, bool *ignore_seq);
+int parse_escape_seq_kbinput(WINDOW *win, int kbinput);
 int get_byte_kbinput(int kbinput);
 #ifdef ENABLE_UTF8
 long add_unicode_digit(int kbinput, long factor, long *uni);
@@ -756,10 +773,10 @@ void reset_cursor(void);
 void edit_draw(const filestruct *fileptr, const char *converted, int
 	line, size_t start);
 void update_line(const filestruct *fileptr, size_t index);
-bool need_horizontal_update(size_t old_pww);
-bool need_vertical_update(size_t old_pww);
+bool need_horizontal_update(size_t pww_save);
+bool need_vertical_update(size_t pww_save);
 void edit_scroll(scroll_dir direction, ssize_t nlines);
-void edit_redraw(const filestruct *old_current, size_t old_pww);
+void edit_redraw(const filestruct *old_current, size_t pww_save);
 void edit_refresh(void);
 void edit_update(update_type location);
 void total_redraw(void);
