@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.115 2006/06/08 02:50:56 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.120 2006/10/29 21:14:53 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -32,7 +32,7 @@
 
 #ifdef ENABLE_NANORC
 
-const static rcoption rcopts[] = {
+static const rcoption rcopts[] = {
     {"boldtext", BOLD_TEXT},
 #ifndef DISABLE_JUSTIFY
     {"brackets", 0},
@@ -108,8 +108,9 @@ static colortype *endcolor = NULL;
 	/* The end of the color list for the current syntax. */
 #endif
 
-/* We have an error in some part of the rcfile.  Put it on stderr and
- * make the user hit Enter to continue starting nano. */
+/* We have an error in some part of the rcfile.  Print the error message
+ * on stderr, and then make the user hit Enter to continue starting
+ * nano. */
 void rcfile_error(const char *msg, ...)
 {
     va_list ap;
@@ -127,8 +128,9 @@ void rcfile_error(const char *msg, ...)
     fprintf(stderr, "\n");
 }
 
-/* Parse the next word from the string.  Return points to '\0' if we hit
- * the end of the line. */
+/* Parse the next word from the string, null-terminate it, and return
+ * a pointer to the first character after the null terminator.  The
+ * returned pointer will point to '\0' if we hit the end of the line. */
 char *parse_next_word(char *ptr)
 {
     while (!isblank(*ptr) && *ptr != '\0')
@@ -202,7 +204,7 @@ char *parse_next_regex(char *ptr)
 	return NULL;
     }
 
-    /* Null terminate and advance ptr. */
+    /* Null-terminate and advance ptr. */
     *ptr++ = '\0';
 
     while (isblank(*ptr))
@@ -834,9 +836,11 @@ void parse_rcfile(FILE *rcstream
 	    rcfile_error(N_("Unknown flag \"%s\""), option);
     }
 
+#ifdef ENABLE_COLOR
     if (endsyntax != NULL && endcolor == NULL)
 	rcfile_error(N_("Syntax \"%s\" has no color commands"),
 		endsyntax->desc);
+#endif
 
     free(buf);
     fclose(rcstream);
@@ -880,10 +884,10 @@ void do_rcfile(void)
 #endif
 		);
 
-#if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
+#ifdef DISABLE_ROOTWRAPPING
     /* We've already read SYSCONFDIR/nanorc, if it's there.  If we're
-     * root and --disable-wrapping-as-root is used, turn wrapping
-     * off now. */
+     * root, and --disable-wrapping-as-root is used, turn wrapping off
+     * now. */
     if (geteuid() == NANO_ROOT_UID)
 	SET(NO_WRAP);
 #endif
