@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.189 2006/05/22 02:08:49 dolorous Exp $ */
+/* $Id: search.c,v 1.191 2006/08/26 16:50:51 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -421,7 +421,7 @@ void do_search(void)
 {
     filestruct *fileptr = openfile->current;
     size_t fileptr_x = openfile->current_x;
-    size_t old_pww = openfile->placewewant;
+    size_t pww_save = openfile->placewewant;
     int i;
     bool didfind;
 
@@ -493,7 +493,7 @@ void do_search(void)
     }
 
     openfile->placewewant = xplustabs();
-    edit_redraw(fileptr, old_pww);
+    edit_redraw(fileptr, pww_save);
     search_replace_abort();
 }
 
@@ -503,7 +503,7 @@ void do_research(void)
 {
     filestruct *fileptr = openfile->current;
     size_t fileptr_x = openfile->current_x;
-    size_t old_pww = openfile->placewewant;
+    size_t pww_save = openfile->placewewant;
     bool didfind;
 
     search_init_globals();
@@ -555,7 +555,7 @@ void do_research(void)
         statusbar(_("No current search pattern"));
 
     openfile->placewewant = xplustabs();
-    edit_redraw(fileptr, old_pww);
+    edit_redraw(fileptr, pww_save);
     search_replace_abort();
 }
 #endif
@@ -576,7 +576,7 @@ int replace_regexp(char *string, bool create)
     /* Iterate through the replacement text to handle subexpression
      * replacement using \1, \2, \3, etc. */
     while (*c != '\0') {
-	int num = (int)(*(c + 1) - '0');
+	int num = (*(c + 1) - '0');
 
 	if (*c != '\\' || num < 1 || num > 9 || num >
 		search_regexp.re_nsub) {
