@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.118 2006/06/21 16:54:51 dolorous Exp $ */
+/* $Id: utils.c,v 1.121 2006/10/29 21:14:53 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -35,8 +35,12 @@ int digits(size_t n)
 {
     int i;
 
-    for (i = 1; n >= 10; n /= 10, i++)
-	;
+    if (n == 0)
+	i = 1;
+    else {
+	for (i = 0; n != 0; n /= 10, i++)
+	    ;
+    }
 
     return i;
 }
@@ -228,7 +232,7 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 	*n += MAX_BUF_SIZE;
     }
 
-    /* Null terminate the buffer. */
+    /* Null-terminate the buffer. */
     null_at(lineptr, indx++);
     *n = indx;
 
@@ -425,7 +429,7 @@ size_t get_page_start(size_t column)
 	return column - (COLS - 2);
 }
 
-/* Return the placewewant associated with current_x, i.e, the zero-based
+/* Return the placewewant associated with current_x, i.e. the zero-based
  * column position of the cursor.  The value will be no smaller than
  * current_x. */
 size_t xplustabs(void)
@@ -434,7 +438,7 @@ size_t xplustabs(void)
 }
 
 /* Return the index in s of the character displayed at the given column,
- * i.e, the largest value such that strnlenpt(s, actual_x(s, column)) <=
+ * i.e. the largest value such that strnlenpt(s, actual_x(s, column)) <=
  * column. */
 size_t actual_x(const char *s, size_t column)
 {
