@@ -1,4 +1,4 @@
-/* $Id: winio.c,v 1.536 2006/06/21 20:51:36 dolorous Exp $ */
+/* $Id: winio.c,v 1.574 2006/10/29 21:14:53 dolorous Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -98,10 +98,7 @@ static bool disable_cursorpos = FALSE;
  *   functions of the former are not arbitrary and the functions of the
  *   latter are.)
  * - F16 on FreeBSD console == Shift-Down on rxvt/Eterm; the former is
- *   omitted.  (Same as above.)
- *
- * Note that Center (5) on the numeric keypad with NumLock off can also
- * be the Begin key. */
+ *   omitted.  (Same as above.) */
 
 /* Read in a sequence of keystrokes from win and save them in the
  * keystroke buffer.  This should only be called when the keystroke
@@ -257,10 +254,12 @@ int *get_input(WINDOW *win, size_t input_len)
 #endif
 
     if (key_buffer_len == 0) {
-	if (win != NULL)
+	if (win != NULL) {
 	    get_key_buffer(win);
 
-	if (key_buffer_len == 0)
+	    if (key_buffer_len == 0)
+		return NULL;
+	} else
 	    return NULL;
     }
 
@@ -347,277 +346,320 @@ int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key)
 		    /* One escape: wait for more input. */
 		case 2:
 		    /* Two escapes: wait for more input. */
+		case 3:
+		    /* Three escapes: wait for more input. */
 		    break;
 		default:
-		    /* More than two escapes: reset the escape counter
-		     * and wait for more input. */
-		    escapes = 0;
+		    /* More than three escapes: limit the escape counter
+		     * to no more than two, and wait for more input. */
+		    escapes %= 3;
 	    }
 	    break;
-#if !defined(NANO_TINY) && defined(KEY_RESIZE)
-	/* Since we don't change the default SIGWINCH handler when
-	 * NANO_TINY is defined, KEY_RESIZE is never generated.  Also,
-	 * Slang and SunOS 5.7-5.9 don't support KEY_RESIZE. */
-	case KEY_RESIZE:
-	    break;
-#endif
-#ifdef PDCURSES
-	case KEY_SHIFT_L:
-	case KEY_SHIFT_R:
-	case KEY_CONTROL_L:
-	case KEY_CONTROL_R:
-	case KEY_ALT_L:
-	case KEY_ALT_R:
-	    break;
-#endif
 	default:
 	    switch (escapes) {
 		case 0:
-		    switch (*kbinput) {
-			case KEY_DOWN:
-			    retval = NANO_NEXTLINE_KEY;
-			    break;
-			case KEY_UP:
-			    retval = NANO_PREVLINE_KEY;
-			    break;
-			case KEY_LEFT:
-			    retval = NANO_BACK_KEY;
-			    break;
-			case KEY_RIGHT:
-			    retval = NANO_FORWARD_KEY;
-			    break;
-#ifdef KEY_HOME
-			/* HP-UX 10-11 doesn't support KEY_HOME. */
-			case KEY_HOME:
-			    retval = NANO_HOME_KEY;
-			    break;
-#endif
-			case KEY_BACKSPACE:
-			    retval = NANO_BACKSPACE_KEY;
-			    break;
-			case KEY_DC:
-			    retval = ISSET(REBIND_DELETE) ?
-				NANO_BACKSPACE_KEY : NANO_DELETE_KEY;
-			    break;
-			case KEY_IC:
-			    retval = NANO_INSERTFILE_KEY;
-			    break;
-			case KEY_NPAGE:
-			    retval = NANO_NEXTPAGE_KEY;
-			    break;
-			case KEY_PPAGE:
-			    retval = NANO_PREVPAGE_KEY;
-			    break;
-			case KEY_ENTER:
-			    retval = NANO_ENTER_KEY;
-			    break;
-			case KEY_A1:	/* Home (7) on numeric keypad
-					 * with NumLock off. */
-			    retval = NANO_HOME_KEY;
-			    break;
-			case KEY_A3:	/* PageUp (9) on numeric keypad
-					 * with NumLock off. */
-			    retval = NANO_PREVPAGE_KEY;
-			    break;
-			case KEY_B2:	/* Center (5) on numeric keypad
-					 * with NumLock off. */
-			    break;
-			case KEY_C1:	/* End (1) on numeric keypad
-					 * with NumLock off. */
-			    retval = NANO_END_KEY;
-			    break;
-			case KEY_C3:	/* PageDown (4) on numeric
-					 * keypad with NumLock off. */
-			    retval = NANO_NEXTPAGE_KEY;
-			    break;
-#ifdef KEY_BEG
-			/* Slang doesn't support KEY_BEG. */
-			case KEY_BEG:	/* Center (5) on numeric keypad
-					 * with NumLock off. */
-			    break;
-#endif
-#ifdef KEY_END
-			/* HP-UX 10-11 doesn't support KEY_END. */
-			case KEY_END:
-			    retval = NANO_END_KEY;
-			    break;
-#endif
-#ifdef KEY_SBEG
-			/* Slang doesn't support KEY_SBEG. */
-			case KEY_SBEG:	/* Center (5) on numeric keypad
-					 * with NumLock off. */
-			    break;
-#endif
-#ifdef KEY_SDC
-			/* Slang doesn't support KEY_SDC. */
-			case KEY_SDC:
-			    retval = ISSET(REBIND_DELETE) ?
-				NANO_BACKSPACE_KEY : NANO_DELETE_KEY;
-			    break;
-#endif
-#ifdef KEY_SEND
-			/* HP-UX 10-11 and Slang don't support
-			 * KEY_SEND. */
-			case KEY_SEND:
-			    retval = NANO_END_KEY;
-			    break;
-#endif
-#ifdef KEY_SHOME
-			/* HP-UX 10-11 and Slang don't support
-			 * KEY_SHOME. */
-			case KEY_SHOME:
-			    retval = NANO_HOME_KEY;
-			    break;
-#endif
-#ifdef KEY_SIC
-			/* Slang doesn't support KEY_SIC. */
-			case KEY_SIC:
-			    retval = NANO_INSERTFILE_KEY;
-			    break;
-#endif
-#ifdef KEY_SLEFT
-			/* Slang doesn't support KEY_SLEFT. */
-			case KEY_SLEFT:
-			    retval = NANO_BACK_KEY;
-			    break;
-#endif
-#ifdef KEY_SRIGHT
-			/* Slang doesn't support KEY_SRIGHT. */
-			case KEY_SRIGHT:
-			    retval = NANO_FORWARD_KEY;
-			    break;
-#endif
-#ifdef KEY_SSUSPEND
-			/* Slang doesn't support KEY_SSUSPEND. */
-			case KEY_SSUSPEND:
-			    retval = NANO_SUSPEND_KEY;
-			    break;
-#endif
-#ifdef KEY_SUSPEND
-			/* Slang doesn't support KEY_SUSPEND. */
-			case KEY_SUSPEND:
-			    retval = NANO_SUSPEND_KEY;
-			    break;
-#endif
-			default:
-			    retval = *kbinput;
-			    break;
-		    }
+		    /* One non-escape: normal input mode.  Save the
+		     * non-escape character as the result. */
+		    retval = *kbinput;
 		    break;
 		case 1:
-		    /* One escape followed by a non-escape: escape
-		     * sequence mode.  Reset the escape counter.  If
-		     * there aren't any other keys waiting, we have a
-		     * meta key sequence, so set meta_key to TRUE and
-		     * save the lowercase version of the non-escape
-		     * character as the result.  If there are other keys
-		     * waiting, we have a true escape sequence, so
-		     * interpret it. */
+		    /* Reset the escape counter. */
 		    escapes = 0;
 		    if (get_key_buffer_len() == 0) {
+			/* One escape followed by a non-escape, and
+			 * there aren't any other keystrokes waiting:
+			 * meta key sequence mode.  Set meta_key to
+			 * TRUE, and save the lowercase version of the
+			 * non-escape character as the result. */
 			*meta_key = TRUE;
 			retval = tolower(*kbinput);
-		    } else {
-			bool ignore_seq;
-
-			retval = parse_escape_seq_kbinput(*kbinput,
-				&ignore_seq);
-
-			/* If the escape sequence is unrecognized and
-			 * not ignored, throw it out. */
-			if (retval == ERR && !ignore_seq) {
-			    if (win == edit)
-				statusbar(_("Unknown Command"));
-			    beep();
-			}
-		    }
+		    } else
+			/* One escape followed by a non-escape, and
+			 * there are other keystrokes waiting: escape
+			 * sequence mode.  Interpret the escape
+			 * sequence. */
+			retval = parse_escape_seq_kbinput(win,
+				*kbinput);
 		    break;
 		case 2:
-		    /* Two escapes followed by one or more decimal
-		     * digits: byte sequence mode.  If the byte
-		     * sequence's range is limited to 2XX (the first
-		     * digit is in the '0' to '2' range and it's the
-		     * first digit, or it's in the '0' to '9' range and
-		     * it's not the first digit), increment the byte
-		     * sequence counter and interpret the digit.  If the
-		     * byte sequence's range is not limited to 2XX, fall
-		     * through. */
-		    if (('0' <= *kbinput && *kbinput <= '2' &&
-			byte_digits == 0) || ('0' <= *kbinput &&
-			*kbinput <= '9' && byte_digits > 0)) {
-			int byte;
+		    if (get_key_buffer_len() == 0) {
+			if (('0' <= *kbinput && *kbinput <= '2' &&
+				byte_digits == 0) || ('0' <= *kbinput &&
+				*kbinput <= '9' && byte_digits > 0)) {
+			    /* Two escapes followed by one or more
+			     * decimal digits, and there aren't any
+			     * other keystrokes waiting: byte sequence
+			     * mode.  If the byte sequence's range is
+			     * limited to 2XX (the first digit is in the
+			     * '0' to '2' range and it's the first
+			     * digit, or it's in the '0' to '9' range
+			     * and it's not the first digit), increment
+			     * the byte sequence counter and interpret
+			     * the digit.  If the byte sequence's range
+			     * is not limited to 2XX, fall through. */
+			    int byte;
 
-			byte_digits++;
-			byte = get_byte_kbinput(*kbinput);
+			    byte_digits++;
+			    byte = get_byte_kbinput(*kbinput);
 
-			if (byte != ERR) {
-			    char *byte_mb;
-			    int byte_mb_len, *seq, i;
+			    if (byte != ERR) {
+				char *byte_mb;
+				int byte_mb_len, *seq, i;
 
-			    /* If we've read in a complete byte
-			     * sequence, reset the byte sequence counter
-			     * and the escape counter, and put back the
-			     * corresponding byte value. */
-			    byte_digits = 0;
+				/* If we've read in a complete byte
+				 * sequence, reset the escape counter
+				 * and the byte sequence counter, and
+				 * put back the corresponding byte
+				 * value. */
+				escapes = 0;
+				byte_digits = 0;
+
+				/* Put back the multibyte equivalent of
+				 * the byte value. */
+				byte_mb = make_mbchar((long)byte,
+					&byte_mb_len);
+
+				seq = (int *)nmalloc(byte_mb_len *
+					sizeof(int));
+
+				for (i = 0; i < byte_mb_len; i++)
+				    seq[i] = (unsigned char)byte_mb[i];
+
+				unget_input(seq, byte_mb_len);
+
+				free(byte_mb);
+				free(seq);
+			    }
+			} else {
+			    /* Reset the escape counter. */
 			    escapes = 0;
-
-			    /* Put back the multibyte equivalent of the
-			     * byte value. */
-			    byte_mb = make_mbchar((long)byte,
-				&byte_mb_len);
-
-			    seq = (int *)nmalloc(byte_mb_len *
-				sizeof(int));
-
-			    for (i = 0; i < byte_mb_len; i++)
-				seq[i] = (unsigned char)byte_mb[i];
-
-			    unget_input(seq, byte_mb_len);
-
-			    free(byte_mb);
-			    free(seq);
+			    if (byte_digits == 0)
+				/* Two escapes followed by a non-decimal
+				 * digit or a decimal digit that would
+				 * create a byte sequence greater than
+				 * 2XX, we're not in the middle of a
+				 * byte sequence, and there aren't any
+				 * other keystrokes waiting: control
+				 * character sequence mode.  Interpret
+				 * the control sequence and save the
+				 * corresponding control character as
+				 * the result. */
+				retval = get_control_kbinput(*kbinput);
+			    else {
+				/* If we're in the middle of a byte
+				 * sequence, reset the byte sequence
+				 * counter and save the character we got
+				 * as the result. */
+				byte_digits = 0;
+				retval = *kbinput;
+			    }
 			}
 		    } else {
-			/* Reset the escape counter. */
+			/* Two escapes followed by a non-escape, and
+			 * there are other keystrokes waiting: combined
+			 * meta and escape sequence mode.  Reset the
+			 * escape counter, set meta_key to TRUE, and
+			 * interpret the escape sequence. */
 			escapes = 0;
-			if (byte_digits == 0)
-			    /* Two escapes followed by a non-decimal
-			     * digit or a decimal digit that would
-			     * create a byte sequence greater than 2XX,
-			     * and we're not in the middle of a byte
-			     * sequence: control character sequence
-			     * mode.  Interpret the control sequence and
-			     * save the corresponding control character
-			     * as the result. */
-			    retval = get_control_kbinput(*kbinput);
-			else {
-			    /* If we're in the middle of a byte
-			     * sequence, reset the byte sequence counter
-			     * and save the character we got as the
-			     * result. */
-			    byte_digits = 0;
-			    retval = *kbinput;
-			}
+			*meta_key = TRUE;
+			retval = parse_escape_seq_kbinput(win,
+				*kbinput);
 		    }
 		    break;
+		case 3:
+		    /* Reset the escape counter. */
+		    escapes = 0;
+		    if (get_key_buffer_len() == 0)
+			/* Three escapes followed by a non-escape, and
+			 * there aren't any other keystrokes waiting:
+			 * normal input mode.  Save the non-escape
+			 * character as the result. */
+			retval = *kbinput;
+		    else
+			/* Three escapes followed by a non-escape, and
+			 * there are other keystrokes waiting: combined
+			 * control character and escape sequence mode.
+			 * Interpret the escape sequence, and interpret
+			 * the result as a control sequence. */
+			retval = get_control_kbinput(
+				parse_escape_seq_kbinput(win,
+				*kbinput));
 	    }
     }
 
     if (retval != ERR) {
-	/* If our result is NANO_CONTROL_8, translate it to either
-	 * Delete or Backspace, depending on whether REBIND_DELETE is
-	 * TRUE or FALSE. */
-	if (retval == NANO_CONTROL_8)
-	    retval = ISSET(REBIND_DELETE) ? NANO_DELETE_KEY :
-		NANO_BACKSPACE_KEY;
+	switch (retval) {
+	    case NANO_CONTROL_8:
+		retval = ISSET(REBIND_DELETE) ? NANO_DELETE_KEY :
+			NANO_BACKSPACE_KEY;
+		break;
+	    case KEY_DOWN:
+		retval = NANO_NEXTLINE_KEY;
+		break;
+	    case KEY_UP:
+		retval = NANO_PREVLINE_KEY;
+		break;
+	    case KEY_LEFT:
+		retval = NANO_BACK_KEY;
+		break;
+	    case KEY_RIGHT:
+		retval = NANO_FORWARD_KEY;
+		break;
+#ifdef KEY_HOME
+	    /* HP-UX 10-11 doesn't support KEY_HOME. */
+	    case KEY_HOME:
+		retval = NANO_HOME_KEY;
+		break;
+#endif
+	    case KEY_BACKSPACE:
+		retval = NANO_BACKSPACE_KEY;
+		break;
+	    case KEY_DC:
+		retval = ISSET(REBIND_DELETE) ? NANO_BACKSPACE_KEY :
+			NANO_DELETE_KEY;
+		break;
+	    case KEY_IC:
+		retval = NANO_INSERTFILE_KEY;
+		break;
+	    case KEY_NPAGE:
+		retval = NANO_NEXTPAGE_KEY;
+		break;
+	    case KEY_PPAGE:
+		retval = NANO_PREVPAGE_KEY;
+		break;
+	    case KEY_ENTER:
+		retval = NANO_ENTER_KEY;
+		break;
+	    case KEY_A1:	/* Home (7) on numeric keypad with
+				 * NumLock off. */
+		retval = NANO_HOME_KEY;
+		break;
+	    case KEY_A3:	/* PageUp (9) on numeric keypad with
+				 * NumLock off. */
+		retval = NANO_PREVPAGE_KEY;
+		break;
+	    case KEY_B2:	/* Center (5) on numeric keypad with
+				 * NumLock off. */
+		retval = ERR;
+		break;
+	    case KEY_C1:	/* End (1) on numeric keypad with
+				 * NumLock off. */
+		retval = NANO_END_KEY;
+		break;
+	    case KEY_C3:	/* PageDown (4) on numeric keypad with
+				 * NumLock off. */
+		retval = NANO_NEXTPAGE_KEY;
+		break;
+#ifdef KEY_BEG
+	    /* Slang doesn't support KEY_BEG. */
+	    case KEY_BEG:	/* Center (5) on numeric keypad with
+				 * NumLock off. */
+		retval = ERR;
+		break;
+#endif
+#ifdef KEY_END
+	    /* HP-UX 10-11 doesn't support KEY_END. */
+	    case KEY_END:
+		retval = NANO_END_KEY;
+		break;
+#endif
+#ifdef KEY_SBEG
+	    /* Slang doesn't support KEY_SBEG. */
+	    case KEY_SBEG:	/* Center (5) on numeric keypad with
+				 * NumLock off. */
+		retval = ERR;
+		break;
+#endif
+#ifdef KEY_SDC
+	    /* Slang doesn't support KEY_SDC. */
+	    case KEY_SDC:
+		retval = ISSET(REBIND_DELETE) ? NANO_BACKSPACE_KEY :
+			NANO_DELETE_KEY;
+		break;
+#endif
+#ifdef KEY_SEND
+	    /* HP-UX 10-11 and Slang don't support KEY_SEND. */
+	    case KEY_SEND:
+		retval = NANO_END_KEY;
+		break;
+#endif
+#ifdef KEY_SHOME
+	    /* HP-UX 10-11 and Slang don't support KEY_SHOME. */
+	    case KEY_SHOME:
+		retval = NANO_HOME_KEY;
+		break;
+#endif
+#ifdef KEY_SIC
+	    /* Slang doesn't support KEY_SIC. */
+	    case KEY_SIC:
+		retval = NANO_INSERTFILE_KEY;
+		break;
+#endif
+#ifdef KEY_SDOWN
+	    /* ncurses and Slang don't support KEY_SDOWN. */
+	    case KEY_SDOWN:
+		retval = NANO_NEXTLINE_KEY;
+		break;
+#endif
+#ifdef KEY_SUP
+	    /* ncurses and Slang don't support KEY_SUP. */
+	    case KEY_SUP:
+		retval = NANO_PREVLINE_KEY;
+		break;
+#endif
+#ifdef KEY_SLEFT
+	    /* Slang doesn't support KEY_SLEFT. */
+	    case KEY_SLEFT:
+		retval = NANO_BACK_KEY;
+		break;
+#endif
+#ifdef KEY_SRIGHT
+	    /* Slang doesn't support KEY_SRIGHT. */
+	    case KEY_SRIGHT:
+		retval = NANO_FORWARD_KEY;
+		break;
+#endif
+#ifdef KEY_SSUSPEND
+	    /* Slang doesn't support KEY_SSUSPEND. */
+	    case KEY_SSUSPEND:
+		retval = NANO_SUSPEND_KEY;
+		break;
+#endif
+#ifdef KEY_SUSPEND
+	    /* Slang doesn't support KEY_SUSPEND. */
+	    case KEY_SUSPEND:
+		retval = NANO_SUSPEND_KEY;
+		break;
+#endif
+#ifdef PDCURSES
+	    case KEY_SHIFT_L:
+	    case KEY_SHIFT_R:
+	    case KEY_CONTROL_L:
+	    case KEY_CONTROL_R:
+	    case KEY_ALT_L:
+	    case KEY_ALT_R:
+		retval = ERR;
+		break;
+#endif
+#if !defined(NANO_TINY) && defined(KEY_RESIZE)
+	    /* Since we don't change the default SIGWINCH handler when
+	     * NANO_TINY is defined, KEY_RESIZE is never generated.
+	     * Also, Slang and SunOS 5.7-5.9 don't support
+	     * KEY_RESIZE. */
+	    case KEY_RESIZE:
+		retval = ERR;
+		break;
+#endif
+	}
 
-	/* If our result is an extended keypad value (i.e, a value
+	/* If our result is an extended keypad value (i.e. a value
 	 * outside of byte range), set func_key to TRUE. */
-	*func_key = !is_byte(retval);
+	if (retval != ERR)
+	    *func_key = !is_byte(retval);
     }
 
 #ifdef DEBUG
-    fprintf(stderr, "parse_kbinput(): kbinput = %d, meta_key = %d, func_key = %d, escapes = %d, byte_digits = %d, retval = %d\n", *kbinput, (int)*meta_key, (int)*func_key, escapes, byte_digits, retval);
+    fprintf(stderr, "parse_kbinput(): kbinput = %d, meta_key = %s, func_key = %s, escapes = %d, byte_digits = %d, retval = %d\n", *kbinput, *meta_key ? "TRUE" : "FALSE", *func_key ? "TRUE" : "FALSE", escapes, byte_digits, retval);
 #endif
 
     /* Return the result. */
@@ -626,18 +668,12 @@ int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key)
 
 /* Translate escape sequences, most of which correspond to extended
  * keypad values, into their corresponding key values.  These sequences
- * are generated when the keypad doesn't support the needed keys.  If
- * the escape sequence is recognized but we want to ignore it, return
- * ERR and set ignore_seq to TRUE; if it's unrecognized, return ERR and
- * set ignore_seq to FALSE.  Assume that Escape has already been read
- * in. */
-int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
-	*ignore_seq)
+ * are generated when the keypad doesn't support the needed keys.
+ * Assume that Escape has already been read in. */
+int get_escape_seq_kbinput(const int *seq, size_t seq_len)
 {
     int retval = ERR;
 
-    *ignore_seq = FALSE;
-
     if (seq_len > 1) {
 	switch (seq[0]) {
 	    case 'O':
@@ -675,7 +711,7 @@ int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
 			break;
 		    case 'E': /* Esc O E == Center (5) on numeric keypad
 			       * with NumLock off on xterm. */
-			*ignore_seq = TRUE;
+			retval = KEY_B2;
 			break;
 		    case 'F': /* Esc O F == End on xterm. */
 			retval = NANO_END_KEY;
@@ -786,7 +822,7 @@ int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
 		    case 'u': /* Esc O u == Center (5) on numeric keypad
 			       * with NumLock off on VT100/VT220/VT320/
 			       * rxvt/Eterm. */
-			*ignore_seq = TRUE;
+			retval = KEY_B2;
 			break;
 		    case 'v': /* Esc O v == Right (6) on numeric keypad
 			       * with NumLock off on VT100/VT220/VT320/
@@ -1002,7 +1038,7 @@ int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
 			break;
 		    case 'E': /* Esc [ E == Center (5) on numeric keypad
 			       * with NumLock off on FreeBSD console. */
-			*ignore_seq = TRUE;
+			retval = KEY_B2;
 			break;
 		    case 'F': /* Esc [ F == End on FreeBSD
 			       * console/Eterm. */
@@ -1126,7 +1162,7 @@ int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
     }
 
 #ifdef DEBUG
-    fprintf(stderr, "get_escape_seq_kbinput(): retval = %d, ignore_seq = %d\n", retval, (int)*ignore_seq);
+    fprintf(stderr, "get_escape_seq_kbinput(): retval = %d\n", retval);
 #endif
 
     return retval;
@@ -1152,11 +1188,9 @@ int get_escape_seq_abcd(int kbinput)
 }
 
 /* Interpret the escape sequence in the keystroke buffer, the first
- * character of which is kbinput.  If we want to ignore the escape
- * sequence, set retval to ERR and ignore_seq to TRUE.  Assume that the
- * keystroke buffer isn't empty, and that the initial escape has already
- * been read in. */
-int parse_escape_seq_kbinput(int kbinput, bool *ignore_seq)
+ * character of which is kbinput.  Assume that the keystroke buffer
+ * isn't empty, and that the initial escape has already been read in. */
+int parse_escape_seq_kbinput(WINDOW *win, int kbinput)
 {
     int retval, *seq;
     size_t seq_len;
@@ -1167,12 +1201,20 @@ int parse_escape_seq_kbinput(int kbinput, bool *ignore_seq)
     unget_input(&kbinput, 1);
     seq_len = get_key_buffer_len();
     seq = get_input(NULL, seq_len);
-    retval = get_escape_seq_kbinput(seq, seq_len, ignore_seq);
+    retval = get_escape_seq_kbinput(seq, seq_len);
 
     free(seq);
 
+    /* If we got an unrecognized escape sequence, throw it out. */
+    if (retval == ERR) {
+	if (win == edit) {
+	    statusbar(_("Unknown Command"));
+	    beep();
+	}
+    }
+
 #ifdef DEBUG
-    fprintf(stderr, "parse_escape_seq_kbinput(): kbinput = %d, ignore_seq = %d, seq_len = %lu, retval = %d\n", kbinput, (int)*ignore_seq, (unsigned long)seq_len, retval);
+    fprintf(stderr, "parse_escape_seq_kbinput(): kbinput = %d, seq_len = %lu, retval = %d\n", kbinput, (unsigned long)seq_len, retval);
 #endif
 
     return retval;
@@ -1520,6 +1562,11 @@ bool get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts)
     if (getmouse(&mevent) == ERR)
 	return FALSE;
 
+    /* If it's not a release or click of the first mouse button, get
+     * out. */
+    if (!(mevent.bstate & (BUTTON1_RELEASED | BUTTON1_CLICKED)))
+	return FALSE;
+
     /* Save the screen coordinates where the mouse event took place. */
     *mouse_x = mevent.x;
     *mouse_y = mevent.y;
@@ -1558,7 +1605,7 @@ bool get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts)
 	    i = COLS / ((currslen / 2) + (currslen % 2));
 
 	/* Calculate the y-coordinate relative to the beginning of
-	 * the shortcut list in bottomwin, i.e, with the sizes of
+	 * the shortcut list in bottomwin, i.e. with the sizes of
 	 * topwin, edit, and the first line of bottomwin subtracted
 	 * out, and set j to it. */
 	j = *mouse_y - (2 - no_more_space()) - editwinrows - 1;
@@ -1615,7 +1662,7 @@ const shortcut *get_shortcut(const shortcut *s_list, int *kbinput, bool
     size_t slen = length_of_list(s_list);
 
 #ifdef DEBUG
-    fprintf(stderr, "get_shortcut(): kbinput = %d, meta_key = %d, func_key = %d\n", *kbinput, (int)*meta_key, (int)*func_key);
+    fprintf(stderr, "get_shortcut(): kbinput = %d, meta_key = %s, func_key = %s\n", *kbinput, *meta_key ? "TRUE" : "FALSE", *func_key ? "TRUE" : "FALSE");
 #endif
 
     /* Check for shortcuts. */
@@ -1630,9 +1677,9 @@ const shortcut *get_shortcut(const shortcut *s_list, int *kbinput, bool
 	 *    shortcut list. */
 
 	if (*kbinput != NANO_NO_KEY && (*kbinput == s->ctrlval ||
-		(*meta_key == TRUE && (*kbinput == s->metaval ||
-		*kbinput == s->miscval)) || (*func_key == TRUE &&
-		*kbinput == s->funcval))) {
+		(*meta_key && (*kbinput == s->metaval || *kbinput ==
+		s->miscval)) || (*func_key && *kbinput ==
+		s->funcval))) {
 	    break;
 	}
 
@@ -1668,7 +1715,7 @@ const toggle *get_toggle(int kbinput, bool meta_key)
     const toggle *t = toggles;
 
 #ifdef DEBUG
-    fprintf(stderr, "get_toggle(): kbinput = %d, meta_key = %d\n", kbinput, (int)meta_key);
+    fprintf(stderr, "get_toggle(): kbinput = %d, meta_key = %s\n", kbinput, meta_key ? "TRUE" : "FALSE");
 #endif
 
     /* Check for toggles. */
@@ -1706,7 +1753,7 @@ void blank_topbar(void)
 	blank_line(topwin, 1, 0, COLS);
 }
 
-/* Blank all the lines of the middle portion of the window, i.e, the
+/* Blank all the lines of the middle portion of the window, i.e. the
  * edit window. */
 void blank_edit(void)
 {
@@ -1804,8 +1851,8 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 
     index = 0;
 
-    if (buf[start_index] != '\t' && (column < start_col || (dollars &&
-	column > 0))) {
+    if (buf[start_index] != '\0' && buf[start_index] != '\t' &&
+	(column < start_col || (dollars && column > 0))) {
 	/* We don't display all of buf[start_index] since it starts to
 	 * the left of the screen. */
 	buf_mb_len = parse_mbchar(buf + start_index, buf_mb, NULL);
@@ -1927,7 +1974,7 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 
     assert(alloc_len >= index + 1);
 
-    /* Null terminate converted. */
+    /* Null-terminate converted. */
     converted[index] = '\0';
 
     /* Make sure converted takes up no more than len columns. */
@@ -1937,12 +1984,12 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
     return converted;
 }
 
-/* Display the path specified in path on the titlebar, along with the
- * current version of nano and whether the current file has been
- * modified.  If path is NULL, assume we're in normal editing mode and
- * display the current filename instead.  Otherwise, assume we're in the
- * file browser, and don't display whether the current file has been
- * modified. */
+/* If path is NULL, we're in normal editing mode, so display the current
+ * version of nano, the current filename, and whether the current file
+ * has been modified on the titlebar.  If path isn't NULL, we're in the
+ * file browser, and path contains the directory to start the file
+ * browser in, so display the current version of nano and the contents
+ * of path on the titlebar. */
 void titlebar(const char *path)
 {
     int space = COLS;
@@ -1959,9 +2006,10 @@ void titlebar(const char *path)
 	 * buffer. */
     size_t statelen = 0;
 	/* The length of the state in columns, or the length of
-	 * "Modified" if the state is blank. */
+	 * "Modified" if the state is blank and we're not in the file
+	 * browser. */
     char *exppath = NULL;
-	/* The file name, expanded for display. */
+	/* The filename, expanded for display. */
     bool newfie = FALSE;
 	/* Do we say "New Buffer"? */
     bool dots = FALSE;
@@ -1970,6 +2018,7 @@ void titlebar(const char *path)
     assert(path != NULL || openfile->filename != NULL);
 
     wattron(topwin, reverse_attr);
+
     blank_titlebar();
 
     /* space has to be at least 4: two spaces before the version message,
@@ -2004,7 +2053,8 @@ void titlebar(const char *path)
 	state = openfile->modified ? _("Modified") : ISSET(VIEW_MODE) ?
 		_("View") : "";
 
-    statelen = strlenpt((state[0] != '\0') ? state : _("Modified"));
+    statelen = strlenpt((state[0] == '\0' && path == NULL) ?
+	_("Modified") : state);
 
     /* If possible, add a space before state. */
     if (space > 0 && statelen < space)
@@ -2030,7 +2080,7 @@ void titlebar(const char *path)
     if (!newfie && prefixlen + statelen < space)
 	prefixlen++;
 
-    /* If we're not in the file browser, path should be the current
+    /* If we're not in the file browser, set path to the current
      * filename. */
     if (path == NULL)
 	path = openfile->filename;
@@ -2045,7 +2095,9 @@ void titlebar(const char *path)
     if (!newfie) {
 	size_t lenpt = strlenpt(path), start_col;
 
-	dots = (lenpt >= space);
+	/* Don't set dots to TRUE if we have fewer than 8 columns (i.e.
+	 * 1 column for padding, plus 7 columns for a filename). */
+	dots = (space >= 8 && lenpt >= space);
 
 	if (dots) {
 	    start_col = lenpt - space + 3;
@@ -2056,7 +2108,19 @@ void titlebar(const char *path)
 	exppath = display_string(path, start_col, space, FALSE);
     }
 
-    if (!dots) {
+    /* If dots is TRUE, we will display something like "File:
+     * ...ename". */
+    if (dots) {
+	mvwaddnstr(topwin, 0, verlen - 1, prefix, actual_x(prefix,
+		prefixlen));
+	if (space <= -3 || newfie)
+	    goto the_end;
+	waddch(topwin, ' ');
+	waddnstr(topwin, "...", space + 3);
+	if (space <= 0)
+	    goto the_end;
+	waddstr(topwin, exppath);
+    } else {
 	size_t exppathlen = newfie ? 0 : strlenpt(exppath);
 	    /* The length of the expanded filename. */
 
@@ -2067,17 +2131,6 @@ void titlebar(const char *path)
 	    waddch(topwin, ' ');
 	    waddstr(topwin, exppath);
 	}
-    } else {
-	/* We will say something like "File: ...ename". */
-	mvwaddnstr(topwin, 0, verlen - 1, prefix, actual_x(prefix,
-		prefixlen));
-	if (space <= -3 || newfie)
-	    goto the_end;
-	waddch(topwin, ' ');
-	waddnstr(topwin, "...", space + 3);
-	if (space <= 0)
-	    goto the_end;
-	waddstr(topwin, exppath);
     }
 
   the_end:
@@ -2319,8 +2372,8 @@ void edit_draw(const filestruct *fileptr, const char *converted, int
 	    int x_start;
 		/* Starting column for mvwaddnstr.  Zero-based. */
 	    int paintlen;
-		/* Number of chars to paint on this line.  There are COLS
-		 * characters on a whole line. */
+		/* Number of chars to paint on this line.  There are
+		 * COLS characters on a whole line. */
 	    size_t index;
 		/* Index in converted where we paint. */
 	    regmatch_t startmatch;
@@ -2453,19 +2506,16 @@ void edit_draw(const filestruct *fileptr, const char *converted, int
 			endmatch.rm_eo <= startpos))
 			goto step_two;
 
-		    /* Now paint the start of fileptr. */
-		    if (end_line != fileptr)
-			/* If the start of fileptr is on a different
-			 * line from the end, paintlen is -1, meaning
-			 * that everything on the line gets painted. */
-			paintlen = -1;
-		    else
-			/* Otherwise, paintlen is the expanded location
-			 * of the end of the match minus the expanded
-			 * location of the beginning of the page. */
-			paintlen = actual_x(converted,
-				strnlenpt(fileptr->data,
-				endmatch.rm_eo) - start);
+		    /* Now paint the start of fileptr.  If the start of
+		     * fileptr is on a different line from the end,
+		     * paintlen is -1, meaning that everything on the
+		     * line gets painted.  Otherwise, paintlen is the
+		     * expanded location of the end of the match minus
+		     * the expanded location of the beginning of the
+		     * page. */
+		    paintlen = (end_line != fileptr) ? -1 :
+			actual_x(converted, strnlenpt(fileptr->data,
+			endmatch.rm_eo) - start);
 
 		    mvwaddnstr(edit, line, 0, converted, paintlen);
 
@@ -2564,8 +2614,8 @@ void edit_draw(const filestruct *fileptr, const char *converted, int
 	int x_start;
 	    /* Starting column for mvwaddnstr().  Zero-based. */
 	int paintlen;
-	    /* Number of chars to paint on this line.  There are COLS
-	     * characters on a whole line. */
+	    /* Number of characters to paint on this line.  There are
+	     * COLS characters on a whole line. */
 	size_t index;
 	    /* Index in converted where we paint. */
 
@@ -2584,17 +2634,13 @@ void edit_draw(const filestruct *fileptr, const char *converted, int
 	     * mark minus the beginning of the page. */
 	    x_start = strnlenpt(fileptr->data, top_x) - start;
 
-	    if (bot_x >= endpos)
-		/* If the end of the mark is off the page, paintlen is
-		 * -1, meaning that everything on the line gets
-		 * painted. */
-		paintlen = -1;
-	    else
-		/* Otherwise, paintlen is the expanded location of the
-		 * end of the mark minus the expanded location of the
-		 * beginning of the mark. */
-		paintlen = strnlenpt(fileptr->data, bot_x) -
-			(x_start + start);
+	    /* If the end of the mark is off the page, paintlen is -1,
+	     * meaning that everything on the line gets painted.
+	     * Otherwise, paintlen is the expanded location of the end
+	     * of the mark minus the expanded location of the beginning
+	     * of the mark. */
+	    paintlen = (bot_x >= endpos) ? -1 : strnlenpt(fileptr->data,
+		bot_x) - (x_start + start);
 
 	    /* If x_start is before the beginning of the page, shift
 	     * paintlen x_start characters to compensate, and put
@@ -2662,37 +2708,37 @@ void update_line(const filestruct *fileptr, size_t index)
 }
 
 /* Return TRUE if we need an update after moving horizontally, and FALSE
- * otherwise.  We need one if the mark is on or if old_pww and
+ * otherwise.  We need one if the mark is on or if pww_save and
  * placewewant are on different pages. */
-bool need_horizontal_update(size_t old_pww)
+bool need_horizontal_update(size_t pww_save)
 {
     return
 #ifndef NANO_TINY
 	openfile->mark_set ||
 #endif
-	get_page_start(old_pww) !=
+	get_page_start(pww_save) !=
 	get_page_start(openfile->placewewant);
 }
 
 /* Return TRUE if we need an update after moving vertically, and FALSE
- * otherwise.  We need one if the mark is on or if old_pww and
+ * otherwise.  We need one if the mark is on or if pww_save and
  * placewewant are on different pages. */
-bool need_vertical_update(size_t old_pww)
+bool need_vertical_update(size_t pww_save)
 {
     return
 #ifndef NANO_TINY
 	openfile->mark_set ||
 #endif
-	get_page_start(old_pww) !=
+	get_page_start(pww_save) !=
 	get_page_start(openfile->placewewant);
 }
 
 /* Scroll the edit window in the given direction and the given number
  * of lines, and draw new lines on the blank lines left after the
- * scrolling.  direction is the direction to scroll, either UP or DOWN,
- * and nlines is the number of lines to scroll.  We change edittop, and
- * assume that current and current_x are up to date.  We also assume
- * that scrollok(edit) is FALSE. */
+ * scrolling.  direction is the direction to scroll, either UP_DIR or
+ * DOWN_DIR, and nlines is the number of lines to scroll.  We change
+ * edittop, and assume that current and current_x are up to date.  We
+ * also assume that scrollok(edit) is FALSE. */
 void edit_scroll(scroll_dir direction, ssize_t nlines)
 {
     bool do_redraw = need_vertical_update(0);
@@ -2710,7 +2756,7 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
      * value of direction) nlines lines, or as many lines as we can if
      * there are fewer than nlines lines available. */
     for (i = nlines; i > 0; i--) {
-	if (direction == UP) {
+	if (direction == UP_DIR) {
 	    if (openfile->edittop == openfile->fileage)
 		break;
 	    openfile->edittop = openfile->edittop->prev;
@@ -2738,7 +2784,7 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
     /* Scroll the text of the edit window up or down nlines lines,
      * depending on the value of direction. */
     scrollok(edit, TRUE);
-    wscrl(edit, (direction == UP) ? -nlines : nlines);
+    wscrl(edit, (direction == UP_DIR) ? -nlines : nlines);
     scrollok(edit, FALSE);
 
     /* Part 2: nlines is the number of lines in the scrolled region of
@@ -2746,9 +2792,10 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
 
     /* If the top or bottom line of the file is now visible in the edit
      * window, we need to draw the entire edit window. */
-    if ((direction == UP && openfile->edittop == openfile->fileage) ||
-	(direction == DOWN && openfile->edittop->lineno + editwinrows -
-	1 >= openfile->filebot->lineno))
+    if ((direction == UP_DIR && openfile->edittop ==
+	openfile->fileage) || (direction == DOWN_DIR &&
+	openfile->edittop->lineno + editwinrows - 1 >=
+	openfile->filebot->lineno))
 	nlines = editwinrows;
 
     /* If the scrolled region contains only one line, and the line
@@ -2767,7 +2814,7 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
 
     /* If we scrolled down, move down to the line before the scrolled
      * region. */
-    if (direction == DOWN) {
+    if (direction == DOWN_DIR) {
 	for (i = editwinrows - nlines; i > 0 && foo != NULL; i--)
 	    foo = foo->next;
     }
@@ -2778,8 +2825,8 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
      * blank, so we don't need to draw it unless the mark is on or we're
      * not on the first page. */
     for (i = nlines; i > 0 && foo != NULL; i--) {
-	if ((i == nlines && direction == DOWN) || (i == 1 &&
-		direction == UP)) {
+	if ((i == nlines && direction == DOWN_DIR) || (i == 1 &&
+		direction == UP_DIR)) {
 	    if (do_redraw)
 		update_line(foo, (foo == openfile->current) ?
 			openfile->current_x : 0);
@@ -2792,10 +2839,10 @@ void edit_scroll(scroll_dir direction, ssize_t nlines)
 
 /* Update any lines between old_current and current that need to be
  * updated.  Use this if we've moved without changing any text. */
-void edit_redraw(const filestruct *old_current, size_t old_pww)
+void edit_redraw(const filestruct *old_current, size_t pww_save)
 {
     bool do_redraw = need_vertical_update(0) ||
-	need_vertical_update(old_pww);
+	need_vertical_update(pww_save);
     const filestruct *foo = NULL;
 
     /* If either old_current or current is offscreen, scroll the edit
@@ -2847,17 +2894,17 @@ void edit_redraw(const filestruct *old_current, size_t old_pww)
 
 	openfile->edittop = old_edittop;
 
-	/* Update old_current if we're not on the first page and/or
-	 * we're not on the same page as before. */
+	/* Update old_current if we're not on the same page as
+	 * before. */
 	if (do_redraw)
 	    update_line(old_current, 0);
 
 	/* Scroll the edit window up or down until edittop is in range
 	 * of current. */
 	if (nlines < 0)
-	    edit_scroll(UP, -nlines);
+	    edit_scroll(UP_DIR, -nlines);
 	else
-	    edit_scroll(DOWN, nlines);
+	    edit_scroll(DOWN_DIR, nlines);
 
 #ifndef NANO_TINY
 	/* If the mark is on, update all the lines between the old first
@@ -2876,9 +2923,9 @@ void edit_redraw(const filestruct *old_current, size_t old_pww)
 	return;
     }
 
-    /* Update old_current and current if we're not on the first page
-     * and/or we're not on the same page as before.  If the mark is on,
-     * update all the lines between old_current and current too. */
+    /* Update old_current and current if we're not on the same page as
+     * before.  If the mark is on, update all the lines between
+     * old_current and current too. */
     foo = old_current;
 
     while (foo != openfile->current) {
@@ -2934,6 +2981,7 @@ void edit_refresh(void)
 	blank_line(edit, nlines, 0, COLS);
 
     reset_cursor();
+
     wnoutrefresh(edit);
 }
 
